@@ -1,4 +1,4 @@
-/* The GIMP -- an image manipulation program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,11 +28,13 @@ struct _LayerAddMaskDialog
 
   GimpLayer       *layer;
   GimpAddMaskType  add_mask_type;
+  GimpChannel     *channel;
   gboolean         invert;
 };
 
 
 LayerAddMaskDialog * layer_add_mask_dialog_new (GimpLayer       *layer,
+                                                GimpContext     *context,
                                                 GtkWidget       *parent,
                                                 GimpAddMaskType  add_mask_type,
                                                 gboolean         invert);
