@@ -1,4 +1,4 @@
-/* The GIMP -- an image manipulation program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
  * GimpVectors Import
@@ -56,13 +56,15 @@
 
 typedef struct
 {
-  GQueue    *stack;
-  GimpImage *image;
-  gboolean   scale;
-  gint       svg_depth;
+  GQueue       *stack;
+  GimpImage    *image;
+  gboolean      scale;
+  gint          svg_depth;
 } SvgParser;
 
+
 typedef struct _SvgHandler SvgHandler;
+
 struct _SvgHandler
 {
   const gchar  *name;
@@ -81,6 +83,7 @@ struct _SvgHandler
   GimpMatrix3  *transform;
 };
 
+
 typedef struct
 {
   gchar        *id;
@@ -95,6 +98,7 @@ static gboolean  gimp_vectors_import  (GimpImage            *image,
                                        gboolean              merge,
                                        gboolean              scale,
                                        gint                  position,
+                                       GList               **ret_vectors,
                                        GError              **error);
 
 static void  svg_parser_start_element (GMarkupParseContext  *context,
@@ -187,7 +191,7 @@ static GList    * parse_path_data     (const gchar  *data);
  *
  * Imports one or more paths and basic shapes from a SVG file.
  *
- * Return value: %TRUE on success, %FALSE if an error occured
+ * Return value: %TRUE on success, %FALSE if an error occurred
  **/
 gboolean
 gimp_vectors_import_file (GimpImage    *image,
@@ -195,14 +199,16 @@ gimp_vectors_import_file (GimpImage    *image,
                           gboolean      merge,
                           gboolean      scale,
                           gint          position,
+                          GList       **ret_vectors,
                           GError      **error)
 {
   g_return_val_if_fail (GIMP_IS_IMAGE (image), FALSE);
   g_return_val_if_fail (filename != NULL, FALSE);
+  g_return_val_if_fail (ret_vectors == NULL || *ret_vectors == NULL, FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   return gimp_vectors_import (image, filename, NULL, 0, merge, scale, position,
-                              error);
+                              ret_vectors, error);
 }
 
 /**
@@ -216,7 +222,7 @@ gimp_vectors_import_file (GimpImage    *image,
  *
  * Imports one or more paths and basic shapes from a SVG file.
  *
- * Return value: %TRUE on success, %FALSE if an error occured
+ * Return value: %TRUE on success, %FALSE if an error occurred
  **/
 gboolean
 gimp_vectors_import_buffer (GimpImage    *image,
@@ -225,14 +231,16 @@ gimp_vectors_import_buffer (GimpImage    *image,
                             gboolean      merge,
                             gboolean      scale,
                             gint          position,
+                            GList       **ret_vectors,
                             GError      **error)
 {
   g_return_val_if_fail (GIMP_IS_IMAGE (image), FALSE);
   g_return_val_if_fail (buffer != NULL || len == 0, FALSE);
+  g_return_val_if_fail (ret_vectors == NULL || *ret_vectors == NULL, FALSE);
   g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
   return gimp_vectors_import (image, NULL, buffer, len, merge, scale, position,
-                              error);
+                              ret_vectors, error);
 }
 
 static gboolean
@@ -243,6 +251,7 @@ gimp_vectors_import (GimpImage    *image,
                      gboolean      merge,
                      gboolean      scale,
                      gint          position,
+                     GList       **ret_vectors,
                      GError      **error)
 {
   GimpXmlParser *xml_parser;
@@ -257,7 +266,7 @@ gimp_vectors_import (GimpImage    *image,
   parser.svg_depth = 0;
 
   /*  the base of the stack, defines the size of the view-port  */
-  base = g_new0 (SvgHandler, 1);
+  base = g_slice_new0 (SvgHandler);
   base->name   = "image";
   base->width  = image->width;
   base->height = image->height;
@@ -299,6 +308,9 @@ gimp_vectors_import (GimpImage    *image,
                   gimp_image_add_vectors (image, vectors, position);
                   gimp_vectors_freeze (vectors);
 
+                  if (ret_vectors)
+                    *ret_vectors = g_list_prepend (*ret_vectors, vectors);
+
                   if (position != -1)
                     position++;
                 }
@@ -347,16 +359,20 @@ gimp_vectors_import (GimpImage    *image,
           SvgPath *path = paths->data;
           GList   *list;
 
+          g_free (path->id);
+
           for (list = path->strokes; list; list = list->next)
             g_object_unref (list->data);
 
-          g_free (path->id);
           g_list_free (path->strokes);
+
+          g_slice_free (SvgPath, path);
         }
 
       g_list_free (base->paths);
-      g_free (base->transform);
-      g_free (base);
+
+      g_slice_free (GimpMatrix3, base->transform);
+      g_slice_free (SvgHandler, base);
     }
 
   g_queue_free (parser.stack);
@@ -377,7 +393,7 @@ svg_parser_start_element (GMarkupParseContext  *context,
   SvgHandler *base;
   gint        i = 0;
 
-  handler = g_new0 (SvgHandler, 1);
+  handler = g_slice_new0 (SvgHandler);
   base    = g_queue_peek_head (parser->stack);
 
   /* if the element is not rendered, always use the generic handler */
@@ -435,14 +451,14 @@ svg_parser_end_element (GMarkupParseContext  *context,
                                        handler->transform);
             }
 
-          g_free (handler->transform);
+          g_slice_free (GimpMatrix3, handler->transform);
         }
 
       base = g_queue_peek_head (parser->stack);
       base->paths = g_list_concat (base->paths, handler->paths);
     }
 
-  g_free (handler);
+  g_slice_free (SvgHandler, handler);
 }
 
 static void
@@ -459,34 +475,41 @@ svg_handler_svg_start (SvgHandler   *handler,
   gdouble      w = handler->width;
   gdouble      h = handler->height;
 
-  matrix = g_new (GimpMatrix3, 1);
+  matrix = g_slice_new (GimpMatrix3);
   gimp_matrix3_identity (matrix);
 
   while (*names)
     {
-      if (strcmp (*names, "x") == 0)
+      switch (*names[0])
         {
-          parse_svg_length (*values,
-                            handler->width, parser->image->xresolution, &x);
-        }
-      else if (strcmp (*names, "y") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->height, parser->image->yresolution, &y);
-        }
-      else if (strcmp (*names, "width") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->width, parser->image->xresolution, &w);
-        }
-      else if (strcmp (*names, "height") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->height, parser->image->yresolution, &h);
-        }
-      else if (strcmp (*names, "viewBox") == 0)
-        {
-          viewbox = *values;
+        case 'x':
+          if (strcmp (*names, "x") == 0)
+            parse_svg_length (*values,
+                              handler->width, parser->image->xresolution, &x);
+          break;
+
+        case 'y':
+          if (strcmp (*names, "y") == 0)
+            parse_svg_length (*values,
+                              handler->height, parser->image->yresolution, &y);
+          break;
+
+        case 'w':
+          if (strcmp (*names, "width") == 0)
+            parse_svg_length (*values,
+                              handler->width, parser->image->xresolution, &w);
+          break;
+
+        case 'h':
+          if (strcmp (*names, "height") == 0)
+            parse_svg_length (*values,
+                              handler->height, parser->image->yresolution, &h);
+          break;
+
+        case 'v':
+          if (strcmp (*names, "viewBox") == 0)
+            viewbox = *values;
+          break;
         }
 
       names++;
@@ -543,7 +566,8 @@ svg_handler_group_start (SvgHandler   *handler,
 
           if (parse_svg_transform (*values, &matrix))
             {
-              handler->transform = g_memdup (&matrix, sizeof (GimpMatrix3));
+              handler->transform = g_slice_new (GimpMatrix3);
+              memcpy (handler->transform, &matrix, sizeof (GimpMatrix3));
 
 #ifdef DEBUG_VECTORS_IMPORT
               g_printerr ("transform %s: %g %g %g   %g %g %g   %g %g %g\n",
@@ -572,24 +596,34 @@ svg_handler_path_start (SvgHandler   *handler,
                         const gchar **values,
                         SvgParser    *parser)
 {
-  SvgPath *path = g_new0 (SvgPath, 1);
+  SvgPath *path = g_slice_new0 (SvgPath);
 
   while (*names)
     {
-      if (strcmp (*names, "id") == 0 && !path->id)
+      switch (*names[0])
         {
-          path->id = g_strdup (*values);
-        }
-      else if (strcmp (*names, "d") == 0 && !path->strokes)
-        {
-          path->strokes = parse_path_data (*values);
-        }
-      else if (strcmp (*names, "transform") == 0 && !handler->transform)
-        {
-          GimpMatrix3  matrix;
+        case 'i':
+          if (strcmp (*names, "id") == 0 && !path->id)
+            path->id = g_strdup (*values);
+          break;
 
-          if (parse_svg_transform (*values, &matrix))
-            handler->transform = g_memdup (&matrix, sizeof (GimpMatrix3));
+        case 'd':
+          if (strcmp (*names, "d") == 0 && !path->strokes)
+            path->strokes = parse_path_data (*values);
+          break;
+
+        case 't':
+          if (strcmp (*names, "transform") == 0 && !handler->transform)
+            {
+              GimpMatrix3  matrix;
+
+              if (parse_svg_transform (*values, &matrix))
+                {
+                  handler->transform = g_slice_new (GimpMatrix3);
+                  memcpy (handler->transform, &matrix, sizeof (GimpMatrix3));
+                }
+            }
+          break;
         }
 
       names++;
@@ -605,7 +639,7 @@ svg_handler_rect_start (SvgHandler   *handler,
                         const gchar **values,
                         SvgParser    *parser)
 {
-  SvgPath    *path   = g_new0 (SvgPath, 1);
+  SvgPath    *path   = g_slice_new0 (SvgPath);
   gdouble     x      = 0.0;
   gdouble     y      = 0.0;
   gdouble     width  = 0.0;
@@ -615,52 +649,64 @@ svg_handler_rect_start (SvgHandler   *handler,
 
   while (*names)
     {
-      if (strcmp (*names, "id") == 0 && !path->id)
+      switch (*names[0])
         {
-          path->id = g_strdup (*values);
-        }
-      else if (strcmp (*names, "x") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->width, parser->image->xresolution,
-                            &x);
-        }
-      else if (strcmp (*names, "y") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->height, parser->image->yresolution,
-                            &y);
-        }
-      else if (strcmp (*names, "width") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->width, parser->image->xresolution,
-                            &width);
-        }
-      else if (strcmp (*names, "height") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->height, parser->image->yresolution,
-                            &height);
-        }
-      else if (strcmp (*names, "rx") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->width, parser->image->xresolution,
-                            &rx);
-        }
-      else if (strcmp (*names, "ry") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->height, parser->image->yresolution,
-                            &ry);
-        }
-      else if (strcmp (*names, "transform") == 0 && !handler->transform)
-        {
-          GimpMatrix3  matrix;
+        case 'i':
+          if (strcmp (*names, "id") == 0 && !path->id)
+            path->id = g_strdup (*values);
+          break;
 
-          if (parse_svg_transform (*values, &matrix))
-            handler->transform = g_memdup (&matrix, sizeof (GimpMatrix3));
+        case 'x':
+          if (strcmp (*names, "x") == 0)
+            parse_svg_length (*values,
+                              handler->width, parser->image->xresolution,
+                              &x);
+          break;
+
+        case 'y':
+          if (strcmp (*names, "y") == 0)
+            parse_svg_length (*values,
+                              handler->height, parser->image->yresolution,
+                              &y);
+          break;
+
+        case 'w':
+          if (strcmp (*names, "width") == 0)
+            parse_svg_length (*values,
+                              handler->width, parser->image->xresolution,
+                              &width);
+          break;
+
+        case 'h':
+          if (strcmp (*names, "height") == 0)
+            parse_svg_length (*values,
+                              handler->height, parser->image->yresolution,
+                              &height);
+          break;
+
+        case 'r':
+          if (strcmp (*names, "rx") == 0)
+            parse_svg_length (*values,
+                              handler->width, parser->image->xresolution,
+                              &rx);
+          else if (strcmp (*names, "ry") == 0)
+            parse_svg_length (*values,
+                              handler->height, parser->image->yresolution,
+                              &ry);
+          break;
+
+        case 't':
+          if (strcmp (*names, "transform") == 0 && !handler->transform)
+            {
+              GimpMatrix3  matrix;
+
+              if (parse_svg_transform (*values, &matrix))
+                {
+                  handler->transform = g_slice_new (GimpMatrix3);
+                  memcpy (handler->transform, &matrix, sizeof (GimpMatrix3));
+                }
+            }
+          break;
         }
 
       names++;
@@ -751,53 +797,67 @@ svg_handler_ellipse_start (SvgHandler   *handler,
                            const gchar **values,
                            SvgParser    *parser)
 {
-  SvgPath    *path   = g_new0 (SvgPath, 1);
+  SvgPath    *path   = g_slice_new0 (SvgPath);
   GimpCoords  center = { 0.0, 0.0, 1.0, 0.5, 0.5, 0.5 };
   gdouble     rx     = 0.0;
   gdouble     ry     = 0.0;
 
   while (*names)
     {
-      if (strcmp (*names, "id") == 0 && !path->id)
+      switch (*names[0])
         {
-          path->id = g_strdup (*values);
-        }
-      else if (strcmp (*names, "cx") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->width, parser->image->xresolution,
-                            &center.x);
-        }
-      else if (strcmp (*names, "cy") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->height, parser->image->yresolution,
-                            &center.y);
-        }
-      else if (strcmp (*names, "r") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->width, parser->image->xresolution,
-                            &rx);
-        }
-      else if (strcmp (*names, "rx") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->width, parser->image->xresolution,
-                            &rx);
-        }
-      else if (strcmp (*names, "ry") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->height, parser->image->yresolution,
-                            &ry);
-        }
-      else if (strcmp (*names, "transform") == 0 && !handler->transform)
-        {
-          GimpMatrix3  matrix;
+        case 'i':
+          if (strcmp (*names, "id") == 0 && !path->id)
+            path->id = g_strdup (*values);
+          break;
 
-          if (parse_svg_transform (*values, &matrix))
-            handler->transform = g_memdup (&matrix, sizeof (GimpMatrix3));
+        case 'c':
+          if (strcmp (*names, "cx") == 0)
+            parse_svg_length (*values,
+                              handler->width, parser->image->xresolution,
+                              &center.x);
+          else if (strcmp (*names, "cy") == 0)
+            parse_svg_length (*values,
+                              handler->height, parser->image->yresolution,
+                              &center.y);
+          break;
+
+        case 'r':
+          if (strcmp (*names, "r") == 0)
+            {
+              parse_svg_length (*values,
+                                handler->width, parser->image->xresolution,
+                                &rx);
+              parse_svg_length (*values,
+                                handler->height, parser->image->yresolution,
+                                &ry);
+            }
+          else if (strcmp (*names, "rx") == 0)
+            {
+              parse_svg_length (*values,
+                                handler->width, parser->image->xresolution,
+                                &rx);
+            }
+          else if (strcmp (*names, "ry") == 0)
+            {
+              parse_svg_length (*values,
+                                handler->height, parser->image->yresolution,
+                                &ry);
+            }
+          break;
+
+        case 't':
+          if (strcmp (*names, "transform") == 0 && !handler->transform)
+            {
+              GimpMatrix3  matrix;
+
+              if (parse_svg_transform (*values, &matrix))
+                {
+                  handler->transform = g_slice_new (GimpMatrix3);
+                  memcpy (handler->transform, &matrix, sizeof (GimpMatrix3));
+                }
+            }
+          break;
         }
 
       names++;
@@ -807,7 +867,8 @@ svg_handler_ellipse_start (SvgHandler   *handler,
   if (rx >= 0.0 && ry >= 0.0)
     path->strokes = g_list_prepend (path->strokes,
                                     gimp_bezier_stroke_new_ellipse (&center,
-                                                                    rx, ry));
+                                                                    rx, ry,
+                                                                    0.0));
 
   handler->paths = g_list_prepend (handler->paths, path);
 }
@@ -818,47 +879,54 @@ svg_handler_line_start (SvgHandler   *handler,
                         const gchar **values,
                         SvgParser    *parser)
 {
-  SvgPath    *path  = g_new0 (SvgPath, 1);
+  SvgPath    *path  = g_slice_new0 (SvgPath);
   GimpCoords  start = { 0.0, 0.0, 1.0, 0.5, 0.5, 0.5 };
   GimpCoords  end   = { 0.0, 0.0, 1.0, 0.5, 0.5, 0.5 };
   GimpStroke *stroke;
 
   while (*names)
     {
-      if (strcmp (*names, "id") == 0 && !path->id)
+      switch (*names[0])
         {
-          path->id = g_strdup (*values);
-        }
-      else if (strcmp (*names, "x1") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->width, parser->image->xresolution,
-                            &start.x);
-        }
-      else if (strcmp (*names, "y1") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->height, parser->image->yresolution,
-                            &start.y);
-        }
-      else if (strcmp (*names, "x2") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->width, parser->image->xresolution,
-                            &end.x);
-        }
-      else if (strcmp (*names, "y2") == 0)
-        {
-          parse_svg_length (*values,
-                            handler->height, parser->image->yresolution,
-                            &end.y);
-        }
-      else if (strcmp (*names, "transform") == 0 && !handler->transform)
-        {
-          GimpMatrix3  matrix;
+        case 'i':
+          if (strcmp (*names, "id") == 0 && !path->id)
+            path->id = g_strdup (*values);
+          break;
 
-          if (parse_svg_transform (*values, &matrix))
-            handler->transform = g_memdup (&matrix, sizeof (GimpMatrix3));
+        case 'x':
+          if (strcmp (*names, "x1") == 0)
+            parse_svg_length (*values,
+                              handler->width, parser->image->xresolution,
+                              &start.x);
+          else if (strcmp (*names, "x2") == 0)
+            parse_svg_length (*values,
+                              handler->width, parser->image->xresolution,
+                              &end.x);
+          break;
+
+        case 'y':
+          if (strcmp (*names, "y1") == 0)
+            parse_svg_length (*values,
+                              handler->height, parser->image->yresolution,
+                              &start.y);
+          else if (strcmp (*names, "y2") == 0)
+            parse_svg_length (*values,
+                              handler->height, parser->image->yresolution,
+                              &end.y);
+          break;
+
+        case 't':
+          if (strcmp (*names, "transform") == 0 && !handler->transform)
+            {
+              GimpMatrix3  matrix;
+
+              if (parse_svg_transform (*values, &matrix))
+                {
+                  handler->transform = g_slice_new (GimpMatrix3);
+                  memcpy (handler->transform, &matrix, sizeof (GimpMatrix3));
+                }
+            }
+          break;
         }
 
       names++;
@@ -879,63 +947,75 @@ svg_handler_poly_start (SvgHandler   *handler,
                         const gchar **values,
                         SvgParser    *parser)
 {
-  SvgPath *path   = g_new0 (SvgPath, 1);
+  SvgPath *path   = g_slice_new0 (SvgPath);
   GString *points = NULL;
 
   while (*names)
     {
-      if (strcmp (*names, "id") == 0 && !path->id)
+      switch (*names[0])
         {
-          path->id = g_strdup (*values);
-        }
-      else if (strcmp (*names, "points") == 0 && !points)
-        {
-          const gchar *p = *values;
-          const gchar *m = NULL;
-          const gchar *l = NULL;
-          gint         n = 0;
+        case 'i':
+          if (strcmp (*names, "id") == 0 && !path->id)
+            path->id = g_strdup (*values);
+          break;
 
-          while (*p)
+        case 'p':
+          if (strcmp (*names, "points") == 0 && !points)
             {
-              while (g_ascii_isspace (*p) || *p == ',')
-                p++;
+              const gchar *p = *values;
+              const gchar *m = NULL;
+              const gchar *l = NULL;
+              gint         n = 0;
 
-              switch (n)
+              while (*p)
                 {
-                case 0:
-                  m = p;
-                  break;
-                case 2:
-                  l = p;
-                  break;
+                  while (g_ascii_isspace (*p) || *p == ',')
+                    p++;
+
+                  switch (n)
+                    {
+                    case 0:
+                      m = p;
+                      break;
+                    case 2:
+                      l = p;
+                      break;
+                    }
+
+                  while (*p && !g_ascii_isspace (*p) && *p != ',')
+                    p++;
+
+                  n++;
                 }
 
-              while (*p && !g_ascii_isspace (*p) && *p != ',')
-                p++;
+              if ((n > 3) && (n % 2 == 0))
+                {
+                  points = g_string_sized_new (p - *values + 8);
 
-              n++;
+                  g_string_append_len (points, "M ", 2);
+                  g_string_append_len (points, m, l - m);
+
+                  g_string_append_len (points, "L ", 2);
+                  g_string_append_len (points, l, p - l);
+
+                  if (strcmp (handler->name, "polygon") == 0)
+                    g_string_append_c (points, 'Z');
+                }
             }
+          break;
 
-          if ((n > 3) && (n % 2 == 0))
+        case 't':
+          if (strcmp (*names, "transform") == 0 && !handler->transform)
             {
-              points = g_string_sized_new (p - *values + 8);
+              GimpMatrix3  matrix;
 
-              g_string_append_len (points, "M ", 2);
-              g_string_append_len (points, m, l - m);
-
-              g_string_append_len (points, "L ", 2);
-              g_string_append_len (points, l, p - l);
-
-              if (strcmp (handler->name, "polygon") == 0)
-                g_string_append_c (points, 'Z');
+              if (parse_svg_transform (*values, &matrix))
+                {
+                  handler->transform = g_slice_new (GimpMatrix3);
+                  memcpy (handler->transform, &matrix, sizeof (GimpMatrix3));
+                }
             }
-        }
-      else if (strcmp (*names, "transform") == 0 && !handler->transform)
-        {
-          GimpMatrix3  matrix;
-
-          if (parse_svg_transform (*values, &matrix))
-            handler->transform = g_memdup (&matrix, sizeof (GimpMatrix3));
+          break;
         }
 
       names++;
@@ -1302,137 +1382,137 @@ parse_path_data (const gchar *data)
     {
       c = data[i];
       if (c >= '0' && c <= '9')
-	{
-	  /* digit */
-	  if (in_num)
-	    {
-	      if (in_exp)
-		{
-		  exp = (exp * 10) + c - '0';
-		  exp_wait_sign = FALSE;
-		}
-	      else if (in_frac)
-		val += (frac *= 0.1) * (c - '0');
-	      else
-		val = (val * 10) + c - '0';
-	    }
-	  else
-	    {
-	      in_num = TRUE;
-	      in_frac = FALSE;
-	      in_exp = FALSE;
-	      exp = 0;
-	      exp_sign = 1;
-	      exp_wait_sign = FALSE;
-	      val = c - '0';
-	      sign = 1;
-	    }
-	}
+        {
+          /* digit */
+          if (in_num)
+            {
+              if (in_exp)
+                {
+                  exp = (exp * 10) + c - '0';
+                  exp_wait_sign = FALSE;
+                }
+              else if (in_frac)
+                val += (frac *= 0.1) * (c - '0');
+              else
+                val = (val * 10) + c - '0';
+            }
+          else
+            {
+              in_num = TRUE;
+              in_frac = FALSE;
+              in_exp = FALSE;
+              exp = 0;
+              exp_sign = 1;
+              exp_wait_sign = FALSE;
+              val = c - '0';
+              sign = 1;
+            }
+        }
       else if (c == '.')
-	{
-	  if (!in_num)
-	    {
-	      in_num = TRUE;
-	      val = 0;
-	    }
-	  in_frac = TRUE;
-	  frac = 1;
-	}
+        {
+          if (!in_num)
+            {
+              in_num = TRUE;
+              val = 0;
+            }
+          in_frac = TRUE;
+          frac = 1;
+        }
       else if ((c == 'E' || c == 'e') && in_num)
-	{
-	  in_exp = TRUE;
-	  exp_wait_sign = TRUE;
-	  exp = 0;
-	  exp_sign = 1;
-	}
+        {
+          in_exp = TRUE;
+          exp_wait_sign = TRUE;
+          exp = 0;
+          exp_sign = 1;
+        }
       else if ((c == '+' || c == '-') && in_exp)
-	{
-	  exp_sign = c == '+' ? 1 : -1;
-	}
+        {
+          exp_sign = c == '+' ? 1 : -1;
+        }
       else if (in_num)
-	{
-	  /* end of number */
+        {
+          /* end of number */
 
-	  val *= sign * pow (10, exp_sign * exp);
-	  if (ctx.rel)
-	    {
-	      /* Handle relative coordinates. This switch statement attempts
-		 to determine _what_ the coords are relative to. This is
-		 underspecified in the 12 Apr working draft. */
-	      switch (ctx.cmd)
-		{
-		case 'l':
-		case 'm':
-		case 'c':
-		case 's':
-		case 'q':
-		case 't':
-		  /* rule: even-numbered params are x-relative, odd-numbered
-		     are y-relative */
-		  if ((ctx.param & 1) == 0)
-		    val += ctx.cpx;
-		  else if ((ctx.param & 1) == 1)
-		    val += ctx.cpy;
-		  break;
+          val *= sign * pow (10, exp_sign * exp);
+          if (ctx.rel)
+            {
+              /* Handle relative coordinates. This switch statement attempts
+                 to determine _what_ the coords are relative to. This is
+                 underspecified in the 12 Apr working draft. */
+              switch (ctx.cmd)
+                {
+                case 'l':
+                case 'm':
+                case 'c':
+                case 's':
+                case 'q':
+                case 't':
+                  /* rule: even-numbered params are x-relative, odd-numbered
+                     are y-relative */
+                  if ((ctx.param & 1) == 0)
+                    val += ctx.cpx;
+                  else if ((ctx.param & 1) == 1)
+                    val += ctx.cpy;
+                  break;
 
-		case 'a':
-		  /* rule: sixth and seventh are x and y, rest are not
-		     relative */
-		  if (ctx.param == 5)
-		    val += ctx.cpx;
-		  else if (ctx.param == 6)
-		    val += ctx.cpy;
-		  break;
-		case 'h':
-		  /* rule: x-relative */
-		  val += ctx.cpx;
-		  break;
-		case 'v':
-		  /* rule: y-relative */
-		  val += ctx.cpy;
-		  break;
-		}
-	    }
+                case 'a':
+                  /* rule: sixth and seventh are x and y, rest are not
+                     relative */
+                  if (ctx.param == 5)
+                    val += ctx.cpx;
+                  else if (ctx.param == 6)
+                    val += ctx.cpy;
+                  break;
+                case 'h':
+                  /* rule: x-relative */
+                  val += ctx.cpx;
+                  break;
+                case 'v':
+                  /* rule: y-relative */
+                  val += ctx.cpy;
+                  break;
+                }
+            }
 
-	  ctx.params[ctx.param++] = val;
-	  parse_path_do_cmd (&ctx, FALSE);
-	  in_num = FALSE;
-	}
+          ctx.params[ctx.param++] = val;
+          parse_path_do_cmd (&ctx, FALSE);
+          in_num = FALSE;
+        }
 
       if (c == '\0')
-	break;
+        break;
       else if ((c == '+' || c == '-') && !exp_wait_sign)
-	{
-	  sign = c == '+' ? 1 : -1;
-	  val = 0;
-	  in_num = TRUE;
-	  in_frac = FALSE;
-	  in_exp = FALSE;
-	  exp = 0;
-	  exp_sign = 1;
-	  exp_wait_sign = FALSE;
-	}
+        {
+          sign = c == '+' ? 1 : -1;
+          val = 0;
+          in_num = TRUE;
+          in_frac = FALSE;
+          in_exp = FALSE;
+          exp = 0;
+          exp_sign = 1;
+          exp_wait_sign = FALSE;
+        }
       else if (c == 'z' || c == 'Z')
-	{
-	  if (ctx.param)
-	    parse_path_do_cmd (&ctx, TRUE);
+        {
+          if (ctx.param)
+            parse_path_do_cmd (&ctx, TRUE);
           if (ctx.stroke)
             gimp_stroke_close (ctx.stroke);
-	}
+        }
       else if (c >= 'A' && c <= 'Z' && c != 'E')
-	{
-	  if (ctx.param)
-	    parse_path_do_cmd (&ctx, TRUE);
-	  ctx.cmd = c + 'a' - 'A';
-	  ctx.rel = FALSE;
-	}
+        {
+          if (ctx.param)
+            parse_path_do_cmd (&ctx, TRUE);
+          ctx.cmd = c + 'a' - 'A';
+          ctx.rel = FALSE;
+        }
       else if (c >= 'a' && c <= 'z' && c != 'e')
-	{
-	  if (ctx.param)
-	    parse_path_do_cmd (&ctx, TRUE);
-	  ctx.cmd = c;
-	  ctx.rel = TRUE;
-	}
+        {
+          if (ctx.param)
+            parse_path_do_cmd (&ctx, TRUE);
+          ctx.cmd = c;
+          ctx.rel = TRUE;
+        }
       /* else c _should_ be whitespace or , */
     }
 
@@ -1450,20 +1530,20 @@ parse_path_default_xy (ParsePathContext *ctx,
   if (ctx->rel)
     {
       for (i = ctx->param; i < n_params; i++)
-	{
-	  if (i > 2)
-	    ctx->params[i] = ctx->params[i - 2];
-	  else if (i == 1)
-	    ctx->params[i] = ctx->cpy;
-	  else if (i == 0)
-	    /* we shouldn't get here (ctx->param > 0 as precondition) */
-	    ctx->params[i] = ctx->cpx;
-	}
+        {
+          if (i > 2)
+            ctx->params[i] = ctx->params[i - 2];
+          else if (i == 1)
+            ctx->params[i] = ctx->cpy;
+          else if (i == 0)
+            /* we shouldn't get here (ctx->param > 0 as precondition) */
+            ctx->params[i] = ctx->cpx;
+        }
     }
   else
     {
       for (i = ctx->param; i < n_params; i++)
-	ctx->params[i] = 0.0;
+        ctx->params[i] = 0.0;
     }
 }
 
@@ -1478,8 +1558,8 @@ parse_path_do_cmd (ParsePathContext *ctx,
     case 'm':
       /* moveto */
       if (ctx->param == 2 || final)
-	{
-	  parse_path_default_xy (ctx, 2);
+        {
+          parse_path_default_xy (ctx, 2);
 
           coords.x = ctx->cpx = ctx->rpx = ctx->params[0];
           coords.y = ctx->cpy = ctx->rpy = ctx->params[1];
@@ -1487,132 +1567,138 @@ parse_path_do_cmd (ParsePathContext *ctx,
           ctx->stroke = gimp_bezier_stroke_new_moveto (&coords);
           ctx->strokes = g_list_prepend (ctx->strokes, ctx->stroke);
 
-	  ctx->param = 0;
-	}
+          ctx->param = 0;
+        }
       break;
+
     case 'l':
       /* lineto */
       if (ctx->param == 2 || final)
-	{
-	  parse_path_default_xy (ctx, 2);
+        {
+          parse_path_default_xy (ctx, 2);
 
           coords.x = ctx->cpx = ctx->rpx = ctx->params[0];
           coords.y = ctx->cpy = ctx->rpy = ctx->params[1];
 
           gimp_bezier_stroke_lineto (ctx->stroke, &coords);
 
-	  ctx->param = 0;
-	}
+          ctx->param = 0;
+        }
       break;
+
     case 'c':
       /* curveto */
       if (ctx->param == 6 || final)
-	{
+        {
           GimpCoords ctrl1 = { 0.0, 0.0, 1.0, 0.5, 0.5, 0.5 };
           GimpCoords ctrl2 = { 0.0, 0.0, 1.0, 0.5, 0.5, 0.5 };
 
-	  parse_path_default_xy (ctx, 6);
+          parse_path_default_xy (ctx, 6);
 
-	  ctrl1.x  = ctx->params[0];
-	  ctrl1.y  = ctx->params[1];
-	  ctrl2.x  = ctx->rpx = ctx->params[2];
-	  ctrl2.y  = ctx->rpy = ctx->params[3];
-	  coords.x = ctx->cpx = ctx->params[4];
-	  coords.y = ctx->cpy = ctx->params[5];
+          ctrl1.x  = ctx->params[0];
+          ctrl1.y  = ctx->params[1];
+          ctrl2.x  = ctx->rpx = ctx->params[2];
+          ctrl2.y  = ctx->rpy = ctx->params[3];
+          coords.x = ctx->cpx = ctx->params[4];
+          coords.y = ctx->cpy = ctx->params[5];
 
           gimp_bezier_stroke_cubicto (ctx->stroke, &ctrl1, &ctrl2, &coords);
 
-	  ctx->param = 0;
-	}
+          ctx->param = 0;
+        }
       break;
+
     case 's':
       /* smooth curveto */
       if (ctx->param == 4 || final)
-	{
+        {
           GimpCoords ctrl1 = { 0.0, 0.0, 1.0, 0.5, 0.5, 0.5 };
           GimpCoords ctrl2 = { 0.0, 0.0, 1.0, 0.5, 0.5, 0.5 };
 
-	  parse_path_default_xy (ctx, 4);
+          parse_path_default_xy (ctx, 4);
 
           ctrl1.x  = 2 * ctx->cpx - ctx->rpx;
-	  ctrl1.y  = 2 * ctx->cpy - ctx->rpy;
+          ctrl1.y  = 2 * ctx->cpy - ctx->rpy;
           ctrl2.x  = ctx->rpx = ctx->params[0];
           ctrl2.y  = ctx->rpy = ctx->params[1];
-	  coords.x = ctx->cpx = ctx->params[2];
-	  coords.y = ctx->cpy = ctx->params[3];
+          coords.x = ctx->cpx = ctx->params[2];
+          coords.y = ctx->cpy = ctx->params[3];
 
           gimp_bezier_stroke_cubicto (ctx->stroke, &ctrl1, &ctrl2, &coords);
 
-	  ctx->param = 0;
-	}
+          ctx->param = 0;
+        }
       break;
+
     case 'h':
       /* horizontal lineto */
       if (ctx->param == 1)
-	{
+        {
           coords.x = ctx->cpx = ctx->rpx = ctx->params[0];
           coords.y = ctx->cpy;
 
           gimp_bezier_stroke_lineto (ctx->stroke, &coords);
 
-	  ctx->param = 0;
-	}
+          ctx->param = 0;
+        }
       break;
+
     case 'v':
       /* vertical lineto */
       if (ctx->param == 1)
-	{
+        {
           coords.x = ctx->cpx;
           coords.y = ctx->cpy = ctx->rpy = ctx->params[0];
 
           gimp_bezier_stroke_lineto (ctx->stroke, &coords);
 
-	  ctx->param = 0;
-	}
+          ctx->param = 0;
+        }
       break;
 
     case 'q':
       /* quadratic bezier curveto */
       if (ctx->param == 4 || final)
-	{
+        {
           GimpCoords ctrl = { 0.0, 0.0, 1.0, 0.5, 0.5, 0.5 };
 
-	  parse_path_default_xy (ctx, 4);
+          parse_path_default_xy (ctx, 4);
 
-	  ctrl.x   = ctx->rpx = ctx->params[0];
-	  ctrl.y   = ctx->rpy = ctx->params[1];
-	  coords.x = ctx->cpx = ctx->params[2];
-	  coords.y = ctx->cpy = ctx->params[3];
+          ctrl.x   = ctx->rpx = ctx->params[0];
+          ctrl.y   = ctx->rpy = ctx->params[1];
+          coords.x = ctx->cpx = ctx->params[2];
+          coords.y = ctx->cpy = ctx->params[3];
 
           gimp_bezier_stroke_conicto (ctx->stroke, &ctrl, &coords);
 
-	  ctx->param = 0;
-	}
+          ctx->param = 0;
+        }
       break;
+
     case 't':
       /* truetype quadratic bezier curveto */
       if (ctx->param == 2 || final)
-	{
+        {
           GimpCoords ctrl = { 0.0, 0.0, 1.0, 0.5, 0.5, 0.5 };
 
-	  parse_path_default_xy (ctx, 2);
+          parse_path_default_xy (ctx, 2);
 
           ctrl.x   = ctx->rpx = 2 * ctx->cpx - ctx->rpx;
           ctrl.y   = ctx->rpy = 2 * ctx->cpy - ctx->rpy;
-	  coords.x = ctx->cpx = ctx->params[0];
-	  coords.y = ctx->cpy = ctx->params[1];
+          coords.x = ctx->cpx = ctx->params[0];
+          coords.y = ctx->cpy = ctx->params[1];
 
           gimp_bezier_stroke_conicto (ctx->stroke, &ctrl, &coords);
 
-	  ctx->param = 0;
-	}
+          ctx->param = 0;
+        }
       else if (final)
-	{
-	  if (ctx->param > 2)
-	    {
+        {
+          if (ctx->param > 2)
+            {
               GimpCoords ctrl = { 0.0, 0.0, 1.0, 0.5, 0.5, 0.5 };
 
-	      parse_path_default_xy (ctx, 4);
+              parse_path_default_xy (ctx, 4);
 
               ctrl.x   = ctx->rpx = ctx->params[0];
               ctrl.y   = ctx->rpy = ctx->params[1];
@@ -1620,23 +1706,24 @@ parse_path_do_cmd (ParsePathContext *ctx,
               coords.y = ctx->cpy = ctx->params[3];
 
               gimp_bezier_stroke_conicto (ctx->stroke, &ctrl, &coords);
-	    }
-	  else
-	    {
-	      parse_path_default_xy (ctx, 2);
+            }
+          else
+            {
+              parse_path_default_xy (ctx, 2);
 
               coords.x = ctx->cpx = ctx->rpx = ctx->params[0];
               coords.y = ctx->cpy = ctx->rpy = ctx->params[1];
 
               gimp_bezier_stroke_lineto (ctx->stroke, &coords);
-	    }
+            }
 
-	  ctx->param = 0;
-	}
+          ctx->param = 0;
+        }
       break;
+
     case 'a':
       if (ctx->param == 7 || final)
-	{
+        {
           coords.x = ctx->cpx = ctx->rpx = ctx->params[5];
           coords.y = ctx->cpy = ctx->rpy = ctx->params[6];
 
@@ -1645,9 +1732,10 @@ parse_path_do_cmd (ParsePathContext *ctx,
                                     gimp_deg_to_rad (ctx->params[2]),
                                     ctx->params[3], ctx->params[4],
                                     &coords);
-	  ctx->param = 0;
-	}
+          ctx->param = 0;
+        }
       break;
+
     default:
       ctx->param = 0;
       break;
