@@ -1,4 +1,4 @@
-/* The GIMP -- an image manipulation program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
  * This program is free software; you can redistribute it and/or modify
@@ -39,12 +39,15 @@
 #include "core/gimpdrawable.h"
 #include "core/gimpgrid.h"
 #include "core/gimpimage.h"
+#include "core/gimpimage-colormap.h"
 #include "core/gimpimage-grid.h"
 #include "core/gimpimage-guides.h"
+#include "core/gimpimage-sample-points.h"
 #include "core/gimplayer.h"
 #include "core/gimplayer-floating-sel.h"
 #include "core/gimplayermask.h"
 #include "core/gimpparasitelist.h"
+#include "core/gimpprogress.h"
 #include "core/gimpselection.h"
 #include "core/gimptemplate.h"
 #include "core/gimpunit.h"
@@ -65,29 +68,30 @@
 
 #include "gimp-intl.h"
 
+
 /* #define GIMP_XCF_PATH_DEBUG */
 
 static gboolean        xcf_load_image_props   (XcfInfo      *info,
-                                               GimpImage    *gimage);
+                                               GimpImage    *image);
 static gboolean        xcf_load_layer_props   (XcfInfo      *info,
-                                               GimpImage    *gimage,
+                                               GimpImage    *image,
                                                GimpLayer    *layer,
                                                gboolean     *apply_mask,
                                                gboolean     *edit_mask,
                                                gboolean     *show_mask,
                                                guint32      *text_layer_flags);
 static gboolean        xcf_load_channel_props (XcfInfo      *info,
-                                               GimpImage    *gimage,
+                                               GimpImage    *image,
                                                GimpChannel **channel);
 static gboolean        xcf_load_prop          (XcfInfo      *info,
                                                PropType     *prop_type,
                                                guint32      *prop_size);
 static GimpLayer     * xcf_load_layer         (XcfInfo      *info,
-                                               GimpImage    *gimage);
+                                               GimpImage    *image);
 static GimpChannel   * xcf_load_channel       (XcfInfo      *info,
-                                               GimpImage    *gimage);
+                                               GimpImage    *image);
 static GimpLayerMask * xcf_load_layer_mask    (XcfInfo      *info,
-                                               GimpImage    *gimage);
+                                               GimpImage    *image);
 static gboolean        xcf_load_hierarchy     (XcfInfo      *info,
                                                TileManager  *tiles);
 static gboolean        xcf_load_level         (XcfInfo      *info,
@@ -99,53 +103,54 @@ static gboolean        xcf_load_tile_rle      (XcfInfo      *info,
                                                gint          data_length);
 static GimpParasite  * xcf_load_parasite      (XcfInfo      *info);
 static gboolean        xcf_load_old_paths     (XcfInfo      *info,
-                                               GimpImage    *gimage);
+                                               GimpImage    *image);
 static gboolean        xcf_load_old_path      (XcfInfo      *info,
-                                               GimpImage    *gimage);
+                                               GimpImage    *image);
 static gboolean        xcf_load_vectors       (XcfInfo      *info,
-                                               GimpImage    *gimage);
+                                               GimpImage    *image);
 static gboolean        xcf_load_vector        (XcfInfo      *info,
-                                               GimpImage    *gimage);
+                                               GimpImage    *image);
 
-#ifdef SWAP_FROM_FILE
-static gboolean        xcf_swap_func          (gint          fd,
-                                               Tile         *tile,
-                                               gint          cmd,
-                                               gpointer      user_data);
-#endif
+
+#define xcf_progress_update(info) G_STMT_START  \
+  {                                             \
+    if (info->progress)                         \
+      gimp_progress_pulse (info->progress);     \
+  } G_STMT_END
 
 
 GimpImage *
 xcf_load_image (Gimp    *gimp,
                 XcfInfo *info)
 {
-  GimpImage    *gimage;
-  GimpLayer    *layer;
-  GimpChannel  *channel;
-  GimpParasite *parasite;
-  guint32       saved_pos;
-  guint32       offset;
-  gint          width;
-  gint          height;
-  gint          image_type;
-  gint          num_successful_elements = 0;
+  GimpImage          *image;
+  GimpLayer          *layer;
+  GimpChannel        *channel;
+  const GimpParasite *parasite;
+  guint32             saved_pos;
+  guint32             offset;
+  gint                width;
+  gint                height;
+  gint                image_type;
+  gint                num_successful_elements = 0;
 
   /* read in the image width, height and type */
   info->cp += xcf_read_int32 (info->fp, (guint32 *) &width, 1);
   info->cp += xcf_read_int32 (info->fp, (guint32 *) &height, 1);
   info->cp += xcf_read_int32 (info->fp, (guint32 *) &image_type, 1);
 
-  gimage = gimp_create_image (gimp,
-                              width, height,
-                              image_type,
-                              FALSE);
+  image = gimp_create_image (gimp, width, height, image_type, FALSE);
+
+  gimp_image_undo_disable (image);
+
+  xcf_progress_update (info);
 
   /* read the image properties */
-  if (!xcf_load_image_props (info, gimage))
+  if (! xcf_load_image_props (info, image))
     goto hard_error;
 
   /* check for a GimpGrid parasite */
-  parasite = gimp_image_parasite_find (GIMP_IMAGE (gimage),
+  parasite = gimp_image_parasite_find (GIMP_IMAGE (image),
                                        gimp_grid_parasite_name ());
   if (parasite)
     {
@@ -153,13 +158,14 @@ xcf_load_image (Gimp    *gimp,
 
       if (grid)
         {
-          gimp_parasite_list_remove (GIMP_IMAGE (gimage)->parasites,
+          gimp_parasite_list_remove (GIMP_IMAGE (image)->parasites,
                                      gimp_parasite_name (parasite));
 
-          gimp_image_set_grid (GIMP_IMAGE (gimage), grid, FALSE);
+          gimp_image_set_grid (GIMP_IMAGE (image), grid, FALSE);
         }
     }
 
+  xcf_progress_update (info);
 
   while (TRUE)
     {
@@ -182,25 +188,18 @@ xcf_load_image (Gimp    *gimp,
         goto error;
 
       /* read in the layer */
-      layer = xcf_load_layer (info, gimage);
+      layer = xcf_load_layer (info, image);
       if (!layer)
         goto error;
 
       num_successful_elements++;
 
+      xcf_progress_update (info);
+
       /* add the layer to the image if its not the floating selection */
       if (layer != info->floating_sel)
-        {
-          gint position = gimp_container_num_children (gimage->layers);
-
-          /*  GIMP 2.2 wants an alpha channel for layers above the
-           *  background layer
-           */
-          if (position > 0)
-            gimp_layer_add_alpha (layer);
-
-          gimp_image_add_layer (gimage, layer, position);
-        }
+        gimp_image_add_layer (image, layer,
+                              gimp_container_num_children (image->layers));
 
       /* restore the saved position so we'll be ready to
        *  read the next offset.
@@ -230,16 +229,18 @@ xcf_load_image (Gimp    *gimp,
         goto error;
 
       /* read in the layer */
-      channel = xcf_load_channel (info, gimage);
+      channel = xcf_load_channel (info, image);
       if (!channel)
         goto error;
 
       num_successful_elements++;
 
+      xcf_progress_update (info);
+
       /* add the channel to the image if its not the selection */
-      if (channel != gimage->selection_mask)
-        gimp_image_add_channel (gimage, channel,
-                                gimp_container_num_children (gimage->channels));
+      if (channel != image->selection_mask)
+        gimp_image_add_channel (image, channel,
+                                gimp_container_num_children (image->channels));
 
       /* restore the saved position so we'll be ready to
        *  read the next offset.
@@ -252,46 +253,52 @@ xcf_load_image (Gimp    *gimp,
     floating_sel_attach (info->floating_sel, info->floating_sel_drawable);
 
   if (info->active_layer)
-    gimp_image_set_active_layer (gimage, info->active_layer);
+    gimp_image_set_active_layer (image, info->active_layer);
 
   if (info->active_channel)
-    gimp_image_set_active_channel (gimage, info->active_channel);
+    gimp_image_set_active_channel (image, info->active_channel);
 
-  gimp_image_set_filename (gimage, info->filename);
+  gimp_image_set_filename (image, info->filename);
 
   if (info->tattoo_state > 0)
-    gimp_image_set_tattoo_state (gimage, info->tattoo_state);
+    gimp_image_set_tattoo_state (image, info->tattoo_state);
 
-  return gimage;
+  gimp_image_undo_enable (image);
+
+  return image;
 
  error:
   if (num_successful_elements == 0)
     goto hard_error;
 
-  g_message ("XCF: This file is corrupt!  I have loaded as much\n"
-             "of it as I can, but it is incomplete.");
+  gimp_message (gimp, G_OBJECT (info->progress), GIMP_MESSAGE_WARNING,
+                _("This XCF file is corrupt!  I have loaded as much "
+                  "of it as I can, but it is incomplete."));
 
-  return gimage;
+  gimp_image_undo_enable (image);
+
+  return image;
 
  hard_error:
-  g_message ("XCF: This file is corrupt!  I could not even\n"
-             "salvage any partial image data from it.");
+  gimp_message (gimp, G_OBJECT (info->progress), GIMP_MESSAGE_ERROR,
+                _("This XCF file is corrupt!  I could not even "
+                  "salvage any partial image data from it."));
 
-  g_object_unref (gimage);
+  g_object_unref (image);
 
   return NULL;
 }
 
 static gboolean
 xcf_load_image_props (XcfInfo   *info,
-                      GimpImage *gimage)
+                      GimpImage *image)
 {
   PropType prop_type;
   guint32  prop_size;
 
   while (TRUE)
     {
-      if (!xcf_load_prop (info, &prop_type, &prop_size))
+      if (! xcf_load_prop (info, &prop_type, &prop_size))
         return FALSE;
 
       switch (prop_type)
@@ -304,41 +311,43 @@ xcf_load_image_props (XcfInfo   *info,
             {
               gint i;
 
-              g_message (_("XCF warning: version 0 of XCF file format\n"
-                           "did not save indexed colormaps correctly.\n"
-                           "Substituting grayscale map."));
+              gimp_message (info->gimp, G_OBJECT (info->progress),
+                            GIMP_MESSAGE_WARNING,
+                            _("XCF warning: version 0 of XCF file format\n"
+                              "did not save indexed colormaps correctly.\n"
+                              "Substituting grayscale map."));
               info->cp +=
-                xcf_read_int32 (info->fp, (guint32 *) &gimage->num_cols, 1);
-              gimage->cmap = g_new (guchar, gimage->num_cols * 3);
-              if (!xcf_seek_pos (info, info->cp + gimage->num_cols, NULL))
+                xcf_read_int32 (info->fp, (guint32 *) &image->num_cols, 1);
+              image->cmap = g_new0 (guchar, GIMP_IMAGE_COLORMAP_SIZE);
+              if (!xcf_seek_pos (info, info->cp + image->num_cols, NULL))
                 return FALSE;
 
-              for (i = 0; i<gimage->num_cols; i++)
+              for (i = 0; i<image->num_cols; i++)
                 {
-                  gimage->cmap[i*3+0] = i;
-                  gimage->cmap[i*3+1] = i;
-                  gimage->cmap[i*3+2] = i;
+                  image->cmap[i*3+0] = i;
+                  image->cmap[i*3+1] = i;
+                  image->cmap[i*3+2] = i;
                 }
             }
           else
             {
               info->cp +=
-                xcf_read_int32 (info->fp, (guint32 *) &gimage->num_cols, 1);
-              gimage->cmap = g_new (guchar, gimage->num_cols * 3);
+                xcf_read_int32 (info->fp, (guint32 *) &image->num_cols, 1);
+              image->cmap = g_new0 (guchar, GIMP_IMAGE_COLORMAP_SIZE);
               info->cp +=
                 xcf_read_int8 (info->fp,
-                               (guint8 *) gimage->cmap, gimage->num_cols * 3);
+                               (guint8 *) image->cmap, image->num_cols * 3);
             }
 
           /* discard color map, if image is not indexed, this is just
-	   * sanity checking to make sure gimp doesn't end up with an
-	   * image state that is impossible.
+           * sanity checking to make sure gimp doesn't end up with an
+           * image state that is impossible.
            */
-          if (gimp_image_base_type (gimage) != GIMP_INDEXED)
+          if (gimp_image_base_type (image) != GIMP_INDEXED)
             {
-              g_free (gimage->cmap);
-              gimage->cmap = NULL;
-              gimage->num_cols = 0;
+              g_free (image->cmap);
+              image->cmap = NULL;
+              image->num_cols = 0;
             }
           break;
 
@@ -353,7 +362,10 @@ xcf_load_image_props (XcfInfo   *info,
                 (compression != COMPRESS_ZLIB) &&
                 (compression != COMPRESS_FRACTAL))
               {
-                g_message ("unknown compression type: %d", (int) compression);
+                gimp_message (info->gimp, G_OBJECT (info->progress),
+                              GIMP_MESSAGE_ERROR,
+                              "Unknown compression type: %d",
+                              (int) compression);
                 return FALSE;
               }
 
@@ -370,8 +382,10 @@ xcf_load_image_props (XcfInfo   *info,
             nguides = prop_size / (4 + 1);
             for (i = 0; i < nguides; i++)
               {
-                info->cp += xcf_read_int32 (info->fp, (guint32 *) &position, 1);
-                info->cp += xcf_read_int8 (info->fp, (guint8 *) &orientation, 1);
+                info->cp += xcf_read_int32 (info->fp,
+                                            (guint32 *) &position, 1);
+                info->cp += xcf_read_int8 (info->fp,
+                                           (guint8 *) &orientation, 1);
 
                 /*  skip -1 guides from old XCFs  */
                 if (position < 0)
@@ -380,15 +394,17 @@ xcf_load_image_props (XcfInfo   *info,
                 switch (orientation)
                   {
                   case XCF_ORIENTATION_HORIZONTAL:
-                    gimp_image_add_hguide (gimage, position, FALSE);
+                    gimp_image_add_hguide (image, position, FALSE);
                     break;
 
                   case XCF_ORIENTATION_VERTICAL:
-                    gimp_image_add_vguide (gimage, position, FALSE);
+                    gimp_image_add_vguide (image, position, FALSE);
                     break;
 
                   default:
-                    g_message ("guide orientation out of range in XCF file");
+                    gimp_message (info->gimp, G_OBJECT (info->progress),
+                                  GIMP_MESSAGE_WARNING,
+                                  "Guide orientation out of range in XCF file");
                     continue;
                   }
               }
@@ -397,7 +413,23 @@ xcf_load_image_props (XcfInfo   *info,
              *  but it restores the list to it's original order, which
              *  cannot be wrong  --Mitch
              */
-            gimage->guides = g_list_reverse (gimage->guides);
+            image->guides = g_list_reverse (image->guides);
+          }
+          break;
+
+        case PROP_SAMPLE_POINTS:
+          {
+            gint32 x, y;
+            gint   i, n_sample_points;
+
+            n_sample_points = prop_size / (4 + 4);
+            for (i = 0; i < n_sample_points; i++)
+              {
+                info->cp += xcf_read_int32 (info->fp, (guint32 *) &x, 1);
+                info->cp += xcf_read_int32 (info->fp, (guint32 *) &y, 1);
+
+                gimp_image_add_sample_point_at_pos (image, x, y, FALSE);
+              }
           }
           break;
 
@@ -410,12 +442,14 @@ xcf_load_image_props (XcfInfo   *info,
             if (xres < GIMP_MIN_RESOLUTION || xres > GIMP_MAX_RESOLUTION ||
                 yres < GIMP_MIN_RESOLUTION || yres > GIMP_MAX_RESOLUTION)
               {
-                g_message ("Warning, resolution out of range in XCF file");
-                xres = gimage->gimp->config->default_image->xresolution;
-                yres = gimage->gimp->config->default_image->yresolution;
+                gimp_message (info->gimp, G_OBJECT (info->progress),
+                              GIMP_MESSAGE_WARNING,
+                              "Warning, resolution out of range in XCF file");
+                xres = image->gimp->config->default_image->xresolution;
+                yres = image->gimp->config->default_image->yresolution;
               }
-            gimage->xresolution = xres;
-            gimage->yresolution = yres;
+            image->xresolution = xres;
+            image->yresolution = yres;
           }
           break;
 
@@ -433,11 +467,13 @@ xcf_load_image_props (XcfInfo   *info,
             while (info->cp - base < prop_size)
               {
                 p = xcf_load_parasite (info);
-                gimp_image_parasite_attach (gimage, p);
+                gimp_image_parasite_attach (image, p);
                 gimp_parasite_free (p);
               }
             if (info->cp - base != prop_size)
-              g_message ("Error while loading an image's parasites");
+              gimp_message (info->gimp, G_OBJECT (info->progress),
+                            GIMP_MESSAGE_WARNING,
+                            "Error while loading an image's parasites");
           }
           break;
 
@@ -448,19 +484,21 @@ xcf_load_image_props (XcfInfo   *info,
             info->cp += xcf_read_int32 (info->fp, &unit, 1);
 
             if ((unit <= GIMP_UNIT_PIXEL) ||
-                (unit >= _gimp_unit_get_number_of_built_in_units (gimage->gimp)))
+                (unit >= _gimp_unit_get_number_of_built_in_units (image->gimp)))
               {
-                g_message ("Warning, unit out of range in XCF file, "
-                           "falling back to inches");
+                gimp_message (info->gimp, G_OBJECT (info->progress),
+                              GIMP_MESSAGE_WARNING,
+                              "Warning, unit out of range in XCF file, "
+                              "falling back to inches");
                 unit = GIMP_UNIT_INCH;
               }
 
-            gimage->resolution_unit = unit;
+            image->resolution_unit = unit;
           }
           break;
 
         case PROP_PATHS:
-          xcf_load_old_paths (info, gimage);
+          xcf_load_old_paths (info, image);
           break;
 
         case PROP_USER_UNIT:
@@ -480,18 +518,18 @@ xcf_load_image_props (XcfInfo   *info,
               if (unit_strings[i] == NULL)
                 unit_strings[i] = g_strdup ("");
 
-            num_units = _gimp_unit_get_number_of_units (gimage->gimp);
+            num_units = _gimp_unit_get_number_of_units (image->gimp);
 
-            for (unit = _gimp_unit_get_number_of_built_in_units (gimage->gimp);
+            for (unit = _gimp_unit_get_number_of_built_in_units (image->gimp);
                  unit < num_units; unit++)
               {
                 /* if the factor and the identifier match some unit
                  * in unitrc, use the unitrc unit
                  */
-                if ((ABS (_gimp_unit_get_factor (gimage->gimp,
+                if ((ABS (_gimp_unit_get_factor (image->gimp,
                                                  unit) - factor) < 1e-5) &&
                     (strcmp (unit_strings[0],
-                             _gimp_unit_get_identifier (gimage->gimp,
+                             _gimp_unit_get_identifier (image->gimp,
                                                         unit)) == 0))
                   {
                     break;
@@ -500,7 +538,7 @@ xcf_load_image_props (XcfInfo   *info,
 
             /* no match */
             if (unit == num_units)
-              unit = _gimp_unit_new (gimage->gimp,
+              unit = _gimp_unit_new (image->gimp,
                                      unit_strings[0],
                                      factor,
                                      digits,
@@ -509,7 +547,7 @@ xcf_load_image_props (XcfInfo   *info,
                                      unit_strings[3],
                                      unit_strings[4]);
 
-            gimage->resolution_unit = unit;
+            image->resolution_unit = unit;
 
             for (i = 0; i < 5; i++)
               g_free (unit_strings[i]);
@@ -520,7 +558,7 @@ xcf_load_image_props (XcfInfo   *info,
           {
             guint32 base = info->cp;
 
-            if (xcf_load_vectors (info, gimage))
+            if (xcf_load_vectors (info, image))
               {
                 if (base + prop_size != info->cp)
                   {
@@ -569,7 +607,7 @@ xcf_load_image_props (XcfInfo   *info,
 
 static gboolean
 xcf_load_layer_props (XcfInfo   *info,
-                      GimpImage *gimage,
+                      GimpImage *image,
                       GimpLayer *layer,
                       gboolean  *apply_mask,
                       gboolean  *edit_mask,
@@ -578,10 +616,11 @@ xcf_load_layer_props (XcfInfo   *info,
 {
   PropType prop_type;
   guint32  prop_size;
+  guint32  value;
 
   while (TRUE)
     {
-      if (!xcf_load_prop (info, &prop_type, &prop_size))
+      if (! xcf_load_prop (info, &prop_type, &prop_size))
         return FALSE;
 
       switch (prop_type)
@@ -631,9 +670,9 @@ xcf_load_layer_props (XcfInfo   *info,
           }
           break;
 
-        case PROP_PRESERVE_TRANSPARENCY:
+        case PROP_LOCK_ALPHA:
           info->cp +=
-            xcf_read_int32 (info->fp, (guint32 *) &layer->preserve_trans, 1);
+            xcf_read_int32 (info->fp, (guint32 *) &layer->lock_alpha, 1);
           break;
 
         case PROP_APPLY_MASK:
@@ -658,7 +697,8 @@ xcf_load_layer_props (XcfInfo   *info,
           break;
 
         case PROP_MODE:
-          info->cp += xcf_read_int32 (info->fp, (guint32 *) &layer->mode, 1);
+          info->cp += xcf_read_int32 (info->fp, &value, 1);
+          layer->mode = value;
           break;
 
         case PROP_TATTOO:
@@ -678,8 +718,11 @@ xcf_load_layer_props (XcfInfo   *info,
                 gimp_item_parasite_attach (GIMP_ITEM (layer), p);
                 gimp_parasite_free (p);
               }
+
             if (info->cp - base != prop_size)
-              g_message ("Error while loading a layer's parasites");
+              gimp_message (info->gimp, G_OBJECT (info->progress),
+                            GIMP_MESSAGE_WARNING,
+                            "Error while loading a layer's parasites");
           }
           break;
 
@@ -716,7 +759,7 @@ xcf_load_layer_props (XcfInfo   *info,
 
 static gboolean
 xcf_load_channel_props (XcfInfo      *info,
-                        GimpImage    *gimage,
+                        GimpImage    *image,
                         GimpChannel **channel)
 {
   PropType prop_type;
@@ -724,7 +767,7 @@ xcf_load_channel_props (XcfInfo      *info,
 
   while (TRUE)
     {
-      if (!xcf_load_prop (info, &prop_type, &prop_size))
+      if (! xcf_load_prop (info, &prop_type, &prop_size))
         return FALSE;
 
       switch (prop_type)
@@ -737,20 +780,19 @@ xcf_load_channel_props (XcfInfo      *info,
           break;
 
         case PROP_SELECTION:
-          g_object_unref (gimage->selection_mask);
-          gimage->selection_mask =
-            gimp_selection_new (gimage,
+          g_object_unref (image->selection_mask);
+          image->selection_mask =
+            gimp_selection_new (image,
                                 gimp_item_width (GIMP_ITEM (*channel)),
                                 gimp_item_height (GIMP_ITEM (*channel)));
-          g_object_ref (gimage->selection_mask);
-          gimp_item_sink (GIMP_ITEM (gimage->selection_mask));
+          g_object_ref_sink (image->selection_mask);
 
-          tile_manager_unref (GIMP_DRAWABLE (gimage->selection_mask)->tiles);
-          GIMP_DRAWABLE (gimage->selection_mask)->tiles =
+          tile_manager_unref (GIMP_DRAWABLE (image->selection_mask)->tiles);
+          GIMP_DRAWABLE (image->selection_mask)->tiles =
             GIMP_DRAWABLE (*channel)->tiles;
           GIMP_DRAWABLE (*channel)->tiles = NULL;
           g_object_unref (*channel);
-          *channel = gimage->selection_mask;
+          *channel = image->selection_mask;
           (*channel)->boundary_known = FALSE;
           (*channel)->bounds_known   = FALSE;
           break;
@@ -819,8 +861,11 @@ xcf_load_channel_props (XcfInfo      *info,
                 gimp_item_parasite_attach (GIMP_ITEM (*channel), p);
                 gimp_parasite_free (p);
               }
+
             if (info->cp - base != prop_size)
-              g_message ("Error while loading a channel's parasites");
+              gimp_message (info->gimp, G_OBJECT (info->progress),
+                            GIMP_MESSAGE_WARNING,
+                            "Error while loading a channel's parasites");
           }
           break;
 
@@ -856,14 +901,22 @@ xcf_load_prop (XcfInfo  *info,
                PropType *prop_type,
                guint32  *prop_size)
 {
-  info->cp += xcf_read_int32 (info->fp, (guint32 *) prop_type, 1);
-  info->cp += xcf_read_int32 (info->fp, (guint32 *) prop_size, 1);
+  if (G_UNLIKELY (xcf_read_int32 (info->fp, (guint32 *) prop_type, 1) != 4))
+    return FALSE;
+
+  info->cp += 4;
+
+  if (G_UNLIKELY (xcf_read_int32 (info->fp, (guint32 *) prop_size, 1) != 4))
+    return FALSE;
+
+  info->cp += 4;
+
   return TRUE;
 }
 
 static GimpLayer *
 xcf_load_layer (XcfInfo   *info,
-                GimpImage *gimage)
+                GimpImage *image)
 {
   GimpLayer     *layer;
   GimpLayerMask *layer_mask;
@@ -893,18 +946,20 @@ xcf_load_layer (XcfInfo   *info,
   info->cp += xcf_read_string (info->fp, &name, 1);
 
   /* create a new layer */
-  layer = gimp_layer_new (gimage, width, height,
+  layer = gimp_layer_new (image, width, height,
                           type, name, 255, GIMP_NORMAL_MODE);
   g_free (name);
   if (! layer)
     return NULL;
 
   /* read in the layer properties */
-  if (! xcf_load_layer_props (info, gimage, layer,
+  if (! xcf_load_layer_props (info, image, layer,
                               &apply_mask, &edit_mask, &show_mask,
                               &text_layer_flags))
     goto error;
 
+  xcf_progress_update (info);
+
   /* call the evil text layer hack that might change our layer pointer */
   active   = (info->active_layer == layer);
   floating = (info->floating_sel == layer);
@@ -931,23 +986,24 @@ xcf_load_layer (XcfInfo   *info,
   if (! xcf_load_hierarchy (info, GIMP_DRAWABLE (layer)->tiles))
     goto error;
 
+  xcf_progress_update (info);
+
   /* read in the layer mask */
   if (layer_mask_offset != 0)
     {
       if (! xcf_seek_pos (info, layer_mask_offset, NULL))
         goto error;
 
-      layer_mask = xcf_load_layer_mask (info, gimage);
+      layer_mask = xcf_load_layer_mask (info, image);
       if (! layer_mask)
         goto error;
 
+      xcf_progress_update (info);
+
       layer_mask->apply_mask = apply_mask;
       layer_mask->edit_mask  = edit_mask;
       layer_mask->show_mask  = show_mask;
 
-      /*  GIMP 2.2 wants an alpha channel for layers with masks  */
-      gimp_layer_add_alpha (layer);
-
       gimp_layer_add_mask (layer, layer_mask, FALSE);
     }
 
@@ -964,7 +1020,7 @@ xcf_load_layer (XcfInfo   *info,
 
 static GimpChannel *
 xcf_load_channel (XcfInfo   *info,
-                  GimpImage *gimage)
+                  GimpImage *image)
 {
   GimpChannel *channel;
   guint32      hierarchy_offset;
@@ -985,15 +1041,17 @@ xcf_load_channel (XcfInfo   *info,
   info->cp += xcf_read_string (info->fp, &name, 1);
 
   /* create a new channel */
-  channel = gimp_channel_new (gimage, width, height, name, &color);
+  channel = gimp_channel_new (image, width, height, name, &color);
   g_free (name);
   if (!channel)
     return NULL;
 
   /* read in the channel properties */
-  if (!xcf_load_channel_props (info, gimage, &channel))
+  if (!xcf_load_channel_props (info, image, &channel))
     goto error;
 
+  xcf_progress_update (info);
+
   /* read the hierarchy and layer mask offsets */
   info->cp += xcf_read_int32 (info->fp, &hierarchy_offset, 1);
 
@@ -1004,6 +1062,8 @@ xcf_load_channel (XcfInfo   *info,
   if (!xcf_load_hierarchy (info, GIMP_DRAWABLE (channel)->tiles))
     goto error;
 
+  xcf_progress_update (info);
+
   if (is_fs_drawable)
     info->floating_sel_drawable = GIMP_DRAWABLE (channel);
 
@@ -1016,7 +1076,7 @@ xcf_load_channel (XcfInfo   *info,
 
 static GimpLayerMask *
 xcf_load_layer_mask (XcfInfo   *info,
-                     GimpImage *gimage)
+                     GimpImage *image)
 {
   GimpLayerMask *layer_mask;
   GimpChannel   *channel;
@@ -1038,16 +1098,18 @@ xcf_load_layer_mask (XcfInfo   *info,
   info->cp += xcf_read_string (info->fp, &name, 1);
 
   /* create a new layer mask */
-  layer_mask = gimp_layer_mask_new (gimage, width, height, name, &color);
+  layer_mask = gimp_layer_mask_new (image, width, height, name, &color);
   g_free (name);
   if (!layer_mask)
     return NULL;
 
   /* read in the layer_mask properties */
   channel = GIMP_CHANNEL (layer_mask);
-  if (!xcf_load_channel_props (info, gimage, &channel))
+  if (!xcf_load_channel_props (info, image, &channel))
     goto error;
 
+  xcf_progress_update (info);
+
   /* read the hierarchy and layer mask offsets */
   info->cp += xcf_read_int32 (info->fp, &hierarchy_offset, 1);
 
@@ -1058,6 +1120,8 @@ xcf_load_layer_mask (XcfInfo   *info,
   if (!xcf_load_hierarchy (info, GIMP_DRAWABLE (layer_mask)->tiles))
     goto error;
 
+  xcf_progress_update (info);
+
   /* attach the floating selection... */
   if (is_fs_drawable)
     info->floating_sel_drawable = GIMP_DRAWABLE (layer_mask);
@@ -1137,10 +1201,10 @@ xcf_load_level (XcfInfo     *info,
   guint32 saved_pos;
   guint32 offset, offset2;
   guint ntiles;
-  gint width;
-  gint height;
-  gint i;
-  gint fail;
+  gint  width;
+  gint  height;
+  gint  i;
+  gint  fail;
   Tile *previous;
   Tile *tile;
 
@@ -1170,7 +1234,9 @@ xcf_load_level (XcfInfo     *info,
 
       if (offset == 0)
         {
-          g_message ("not enough tiles found in level");
+          gimp_message (info->gimp, G_OBJECT (info->progress),
+                        GIMP_MESSAGE_ERROR,
+                        "not enough tiles found in level");
           return FALSE;
         }
 
@@ -1256,7 +1322,8 @@ xcf_load_level (XcfInfo     *info,
 
   if (offset != 0)
     {
-      g_message ("encountered garbage after reading level: %d", offset);
+      gimp_message (info->gimp, G_OBJECT (info->progress), GIMP_MESSAGE_ERROR,
+                    "encountered garbage after reading level: %d", offset);
       return FALSE;
     }
 
@@ -1267,27 +1334,9 @@ static gboolean
 xcf_load_tile (XcfInfo *info,
                Tile    *tile)
 {
-#ifdef SWAP_FROM_FILE
-
-  if (!info->swap_num)
-    {
-      info->ref_count = g_new (int, 1);
-      info->swap_num = tile_swap_add (info->filename,
-                                      xcf_swap_func,
-                                      info->ref_count);
-    }
-
-  tile->swap_num = info->swap_num;
-  tile->swap_offset = info->cp;
-  *info->ref_count += 1;
-
-#else
-
   info->cp += xcf_read_int8 (info->fp, tile_data_pointer(tile, 0, 0),
                              tile_size (tile));
 
-#endif
-
   return TRUE;
 }
 
@@ -1427,9 +1476,12 @@ static GimpParasite *
 xcf_load_parasite (XcfInfo *info)
 {
   GimpParasite *p;
+  gchar        *name;
+
+  info->cp += xcf_read_string (info->fp, &name, 1);
+  p = gimp_parasite_new (name, 0, 0, NULL);
+  g_free (name);
 
-  p = g_new (GimpParasite, 1);
-  info->cp += xcf_read_string (info->fp, &p->name, 1);
   info->cp += xcf_read_int32 (info->fp, &p->flags, 1);
   info->cp += xcf_read_int32 (info->fp, &p->size, 1);
   p->data = g_new (gchar, p->size);
@@ -1440,7 +1492,7 @@ xcf_load_parasite (XcfInfo *info)
 
 static gboolean
 xcf_load_old_paths (XcfInfo   *info,
-                    GimpImage *gimage)
+                    GimpImage *image)
 {
   guint32      num_paths;
   guint32      last_selected_row;
@@ -1450,20 +1502,20 @@ xcf_load_old_paths (XcfInfo   *info,
   info->cp += xcf_read_int32 (info->fp, &num_paths, 1);
 
   while (num_paths-- > 0)
-    xcf_load_old_path (info, gimage);
+    xcf_load_old_path (info, image);
 
   active_vectors = (GimpVectors *)
-    gimp_container_get_child_by_index (gimage->vectors, last_selected_row);
+    gimp_container_get_child_by_index (image->vectors, last_selected_row);
 
   if (active_vectors)
-    gimp_image_set_active_vectors (gimage, active_vectors);
+    gimp_image_set_active_vectors (image, active_vectors);
 
   return TRUE;
 }
 
 static gboolean
 xcf_load_old_path (XcfInfo   *info,
-                   GimpImage *gimage)
+                   GimpImage *image)
 {
   gchar                  *name;
   guint32                 locked;
@@ -1539,7 +1591,7 @@ xcf_load_old_path (XcfInfo   *info,
         }
     }
 
-  vectors = gimp_vectors_compat_new (gimage, name, points, num_points, closed);
+  vectors = gimp_vectors_compat_new (image, name, points, num_points, closed);
 
   g_free (name);
   g_free (points);
@@ -1549,15 +1601,15 @@ xcf_load_old_path (XcfInfo   *info,
   if (tattoo)
     GIMP_ITEM (vectors)->tattoo = tattoo;
 
-  gimp_image_add_vectors (gimage, vectors,
-                          gimp_container_num_children (gimage->vectors));
+  gimp_image_add_vectors (image, vectors,
+                          gimp_container_num_children (image->vectors));
 
   return TRUE;
 }
 
 static gboolean
 xcf_load_vectors (XcfInfo   *info,
-                  GimpImage *gimage)
+                  GimpImage *image)
 {
   guint32      version;
   guint32      active_index;
@@ -1575,7 +1627,9 @@ xcf_load_vectors (XcfInfo   *info,
 
   if (version != 1)
     {
-      g_message ("Unknown vectors version: %d (skipping)", version);
+      gimp_message (info->gimp, G_OBJECT (info->progress),
+                    GIMP_MESSAGE_WARNING,
+                    "Unknown vectors version: %d (skipping)", version);
       return FALSE;
     }
 
@@ -1587,14 +1641,14 @@ xcf_load_vectors (XcfInfo   *info,
 #endif
 
   while (num_paths-- > 0)
-    if (! xcf_load_vector (info, gimage))
+    if (! xcf_load_vector (info, image))
       return FALSE;
 
   active_vectors = (GimpVectors *)
-    gimp_container_get_child_by_index (gimage->vectors, active_index);
+    gimp_container_get_child_by_index (image->vectors, active_index);
 
   if (active_vectors)
-    gimp_image_set_active_vectors (gimage, active_vectors);
+    gimp_image_set_active_vectors (image, active_vectors);
 
 #ifdef GIMP_XCF_PATH_DEBUG
   g_printerr ("xcf_load_vectors: loaded %d bytes\n", info->cp - base);
@@ -1604,7 +1658,7 @@ xcf_load_vectors (XcfInfo   *info,
 
 static gboolean
 xcf_load_vector (XcfInfo   *info,
-                 GimpImage *gimage)
+                 GimpImage *image)
 {
   gchar       *name;
   GimpTattoo   tattoo = 0;
@@ -1632,7 +1686,7 @@ xcf_load_vector (XcfInfo   *info,
               name, tattoo, visible, linked, num_parasites, num_strokes);
 #endif
 
-  vectors = gimp_vectors_new (gimage, name);
+  vectors = gimp_vectors_new (image, name);
 
   GIMP_ITEM (vectors)->visible = visible ? TRUE : FALSE;
   GIMP_ITEM (vectors)->linked  = linked  ? TRUE : FALSE;
@@ -1653,12 +1707,12 @@ xcf_load_vector (XcfInfo   *info,
 
   for (i = 0; i < num_strokes; i++)
     {
-      gint         stroke_type_id;
-      gint         closed;
-      gint         num_axes;
-      gint         num_control_points;
-      gint         type;
-      gfloat       coords[6] = { 0.0, 0.0, 1.0, 0.5, 0.5, 0.5 };
+      guint32      stroke_type_id;
+      guint32      closed;
+      guint32      num_axes;
+      guint32      num_control_points;
+      guint32      type;
+      gfloat       coords[6] = GIMP_COORDS_DEFAULT_VALUES;
       GimpStroke  *stroke;
       gint         j;
 
@@ -1736,69 +1790,8 @@ xcf_load_vector (XcfInfo   *info,
       gimp_vectors_stroke_add (vectors, stroke);
     }
 
-  gimp_image_add_vectors (gimage, vectors,
-                          gimp_container_num_children (gimage->vectors));
+  gimp_image_add_vectors (image, vectors,
+                          gimp_container_num_children (image->vectors));
 
   return TRUE;
 }
-
-#ifdef SWAP_FROM_FILE
-
-static gboolean
-xcf_swap_func (gint      fd,
-               Tile     *tile,
-               gint      cmd,
-               gpointer  user_data)
-{
-  gint bytes;
-  gint err;
-  gint nleft;
-  gint *ref_count;
-
-  switch (cmd)
-    {
-    case SWAP_IN:
-      lseek (fd, tile->swap_offset, SEEK_SET);
-
-      bytes = tile_size (tile);
-      tile_alloc (tile);
-
-      nleft = bytes;
-      while (nleft > 0)
-        {
-          do {
-            err = read (fd, tile->data + bytes - nleft, nleft);
-          } while ((err == -1) && ((errno == EAGAIN) || (errno == EINTR)));
-
-          if (err <= 0)
-            {
-              g_message ("unable to read tile data from xcf file: "
-                         "%d ( %d ) bytes read", err, nleft);
-              return FALSE;
-            }
-
-          nleft -= err;
-        }
-      break;
-
-    case SWAP_OUT:
-    case SWAP_DELETE:
-    case SWAP_COMPRESS:
-      ref_count = user_data;
-      *ref_count -= 1;
-      if (*ref_count == 0)
-        {
-          tile_swap_remove (tile->swap_num);
-          g_free (ref_count);
-        }
-
-      tile->swap_num = 1;
-      tile->swap_offset = -1;
-
-      return TRUE;
-    }
-
-  return FALSE;
-}
-
-#endif
