@@ -24,6 +24,9 @@
 #include <string.h>
 
 #include "gimp.h"
+#undef GIMP_DISABLE_DEPRECATED
+#undef __GIMP_PATTERNS_H__
+#include "gimppatterns.h"
 
 /**
  * gimp_patterns_set_pattern:
