@@ -1,8 +1,8 @@
-/* bmpread.c	reads any bitmap I could get for testing */
+/* bmpread.c    reads any bitmap I could get for testing */
 /* Alexander.Schulz@stud.uni-karlsruhe.de                */
 
 /*
- * The GIMP -- an image manipulation program
+ * GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,11 +24,9 @@
 #include "config.h"
 
 #include <errno.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
 
-#include <gtk/gtk.h>
+#include <glib/gstdio.h>
 
 #include <libgimp/gimp.h>
 
@@ -36,11 +34,12 @@
 
 #include "libgimp/stdplugins-intl.h"
 
+
 #if !defined(WIN32) || defined(__MINGW32__)
-#define BI_RGB 		0
-#define BI_RLE8  	1
-#define BI_RLE4  	2
-#define BI_BITFIELDS  	3
+#define BI_RGB          0
+#define BI_RLE8         1
+#define BI_RLE4         2
+#define BI_BITFIELDS    3
 #endif
 
 static gint32 ReadImage (FILE     *fd,
@@ -69,10 +68,10 @@ ToS (const guchar *puffer)
 
 static gboolean
 ReadColorMap (FILE     *fd,
-	      guchar    buffer[256][3],
-	      gint      number,
-	      gint      size,
-	      gboolean *grey)
+              guchar    buffer[256][3],
+              gint      number,
+              gint      size,
+              gboolean *grey)
 {
   gint   i;
   guchar rgb[4];
@@ -82,10 +81,10 @@ ReadColorMap (FILE     *fd,
   for (i = 0; i < number ; i++)
     {
       if (!ReadOK (fd, rgb, size))
-	{
-	  g_message (_("Bad colormap"));
-	  return FALSE;
-	}
+        {
+          g_message (_("Bad colormap"));
+          return FALSE;
+        }
 
       /* Bitmap save the colors in another order! But change only once! */
 
@@ -98,18 +97,14 @@ ReadColorMap (FILE     *fd,
 }
 
 static gboolean
-ReadChannelMasks (FILE *fd, Bitmap_Channel *masks, guint channels)
+ReadChannelMasks (guint32 *tmp, Bitmap_Channel *masks, guint channels)
 {
-  guint32 tmp[3];
   guint32 mask;
   gint   i, nbits, offset, bit;
-  
-  if (!ReadOK (fd, tmp, 3 * sizeof (guint32)))
-    return FALSE;
 
   for (i = 0; i < channels; i++)
     {
-      mask = ToL ((guchar*) &tmp[i]);
+      mask = tmp[i];
       masks[i].mask = mask;
       nbits = 0;
       offset = -1;
@@ -124,9 +119,11 @@ ReadChannelMasks (FILE *fd, Bitmap_Channel *masks, guint channels)
           mask = mask >> 1;
         }
       masks[i].shiftin = offset;
-      masks[i].shiftout = 8 - nbits;
+      masks[i].max_value = (gfloat)((1<<(nbits))-1);
+
 #ifdef DEBUG
-      g_print ("Channel %d mask %08x in %d out %d\n", i, masks[i].mask, masks[i].shiftin, masks[i].shiftout);
+      g_print ("Channel %d mask %08x in %d max_val %d\n",
+               i, masks[i].mask, masks[i].shiftin, (gint)masks[i].max_value);
 #endif
     }
   return TRUE;
@@ -136,17 +133,16 @@ gint32
 ReadBMP (const gchar *name)
 {
   FILE     *fd;
-  gchar    *temp_buf;
   guchar    buffer[64];
   gint      ColormapSize, rowbytes, Maps;
-  gboolean  Grey;
+  gboolean  Grey = FALSE;
   guchar    ColorMap[256][3];
   gint32    image_ID;
-  guchar    magick[2];
-  Bitmap_Channel *masks;
+  gchar     magick[2];
+  Bitmap_Channel masks[4];
 
   filename = name;
-  fd = fopen (filename, "rb");
+  fd = g_fopen (filename, "rb");
 
   if (!fd)
     {
@@ -155,33 +151,31 @@ ReadBMP (const gchar *name)
       return -1;
     }
 
-  temp_buf = g_strdup_printf (_("Opening '%s'..."),
-                              gimp_filename_to_utf8 (name));
-  gimp_progress_init (temp_buf);
-  g_free (temp_buf);
+  gimp_progress_init_printf (_("Opening '%s'"),
+                             gimp_filename_to_utf8 (name));
 
   /* It is a File. Now is it a Bitmap? Read the shortest possible header */
 
-  if (!ReadOK (fd, magick, 2) || !(!strncmp (magick,"BA",2) ||
-     !strncmp (magick,"BM",2) || !strncmp (magick,"IC",2)   ||
-     !strncmp (magick,"PI",2) || !strncmp (magick,"CI",2)   ||
-     !strncmp (magick,"CP",2)))
+  if (!ReadOK (fd, magick, 2) || !(!strncmp (magick, "BA", 2) ||
+     !strncmp (magick, "BM", 2) || !strncmp (magick, "IC", 2) ||
+     !strncmp (magick, "PI", 2) || !strncmp (magick, "CI", 2) ||
+     !strncmp (magick, "CP", 2)))
     {
       g_message (_("'%s' is not a valid BMP file"),
                  gimp_filename_to_utf8 (filename));
       return -1;
     }
 
-  while (!strncmp(magick,"BA",2))
+  while (!strncmp (magick, "BA", 2))
     {
       if (!ReadOK (fd, buffer, 12))
-	{
+        {
           g_message (_("'%s' is not a valid BMP file"),
                       gimp_filename_to_utf8 (filename));
           return -1;
         }
       if (!ReadOK (fd, magick, 2))
-	{
+        {
           g_message (_("'%s' is not a valid BMP file"),
                       gimp_filename_to_utf8 (filename));
           return -1;
@@ -230,30 +224,105 @@ ReadBMP (const gchar *name)
       Bitmap_Head.biSizeIm  = 0;
       Bitmap_Head.biXPels   = Bitmap_Head.biYPels = 0;
       Bitmap_Head.biClrUsed = 0;
+      Bitmap_Head.biClrImp  = 0;
+      Bitmap_Head.masks[0]  = 0;
+      Bitmap_Head.masks[1]  = 0;
+      Bitmap_Head.masks[2]  = 0;
+      Bitmap_Head.masks[3]  = 0;
+
+      memset(masks, 0, sizeof(masks));
       Maps = 3;
     }
   else if (Bitmap_File_Head.biSize == 40) /* Windows 3.x */
     {
-      if (!ReadOK (fd, buffer, Bitmap_File_Head.biSize - 4))
+      if (!ReadOK (fd, buffer, 36))
         {
           g_message (_("Error reading BMP file header from '%s'"),
                       gimp_filename_to_utf8 (filename));
           return -1;
         }
-      Bitmap_Head.biWidth   = ToL (&buffer[0x00]);	/* 12 */
-      Bitmap_Head.biHeight  = ToL (&buffer[0x04]);	/* 16 */
+      Bitmap_Head.biWidth   = ToL (&buffer[0x00]);      /* 12 */
+      Bitmap_Head.biHeight  = ToL (&buffer[0x04]);      /* 16 */
       Bitmap_Head.biPlanes  = ToS (&buffer[0x08]);       /* 1A */
-      Bitmap_Head.biBitCnt  = ToS (&buffer[0x0A]);	/* 1C */
-      Bitmap_Head.biCompr   = ToL (&buffer[0x0C]);	/* 1E */
-      Bitmap_Head.biSizeIm  = ToL (&buffer[0x10]);	/* 22 */
-      Bitmap_Head.biXPels   = ToL (&buffer[0x14]);	/* 26 */
-      Bitmap_Head.biYPels   = ToL (&buffer[0x18]);	/* 2A */
-      Bitmap_Head.biClrUsed = ToL (&buffer[0x1C]);	/* 2E */
-      Bitmap_Head.biClrImp  = ToL (&buffer[0x20]);	/* 32 */
-    					                /* 36 */
+      Bitmap_Head.biBitCnt  = ToS (&buffer[0x0A]);      /* 1C */
+      Bitmap_Head.biCompr   = ToL (&buffer[0x0C]);      /* 1E */
+      Bitmap_Head.biSizeIm  = ToL (&buffer[0x10]);      /* 22 */
+      Bitmap_Head.biXPels   = ToL (&buffer[0x14]);      /* 26 */
+      Bitmap_Head.biYPels   = ToL (&buffer[0x18]);      /* 2A */
+      Bitmap_Head.biClrUsed = ToL (&buffer[0x1C]);      /* 2E */
+      Bitmap_Head.biClrImp  = ToL (&buffer[0x20]);      /* 32 */
+      Bitmap_Head.masks[0]  = 0;
+      Bitmap_Head.masks[1]  = 0;
+      Bitmap_Head.masks[2]  = 0;
+      Bitmap_Head.masks[3]  = 0;
+
       Maps = 4;
+      memset(masks, 0, sizeof(masks));
+
+      if (Bitmap_Head.biCompr == BI_BITFIELDS)
+      {
+         if (!ReadOK (fd, buffer, 3 * sizeof (guint32)))
+         {
+            g_message (_("Error reading BMP file header from '%s'"),
+                       gimp_filename_to_utf8 (filename));
+            return -1;
+         }
+         Bitmap_Head.masks[0] = ToL(&buffer[0x00]);
+         Bitmap_Head.masks[1] = ToL(&buffer[0x04]);
+         Bitmap_Head.masks[2] = ToL(&buffer[0x08]);
+         ReadChannelMasks (&Bitmap_Head.masks[0], masks, 3);
+      }
+      else
+         switch (Bitmap_Head.biBitCnt)
+         {
+         case 32:
+            masks[0].mask     = 0x00ff0000;
+            masks[0].shiftin  = 16;
+            masks[0].max_value= (gfloat)255.0;
+            masks[1].mask     = 0x0000ff00;
+            masks[1].shiftin  = 8;
+            masks[1].max_value= (gfloat)255.0;
+            masks[2].mask     = 0x000000ff;
+            masks[2].shiftin  = 0;
+            masks[2].max_value= (gfloat)255.0;
+            masks[3].mask     = 0xff000000;
+            masks[3].shiftin  = 24;
+            masks[3].max_value= (gfloat)255.0;
+            break;
+         case 24:
+            masks[0].mask     = 0xff0000;
+            masks[0].shiftin  = 16;
+            masks[0].max_value= (gfloat)255.0;
+            masks[1].mask     = 0x00ff00;
+            masks[1].shiftin  = 8;
+            masks[1].max_value= (gfloat)255.0;
+            masks[2].mask     = 0x0000ff;
+            masks[2].shiftin  = 0;
+            masks[2].max_value= (gfloat)255.0;
+            masks[3].mask     = 0x0;
+            masks[3].shiftin  = 0;
+            masks[3].max_value= (gfloat)0.0;
+            break;
+         case 16:
+            masks[0].mask     = 0x7c00;
+            masks[0].shiftin  = 10;
+            masks[0].max_value= (gfloat)31.0;
+            masks[1].mask     = 0x03e0;
+            masks[1].shiftin  = 5;
+            masks[1].max_value= (gfloat)31.0;
+            masks[2].mask     = 0x001f;
+            masks[2].shiftin  = 0;
+            masks[2].max_value= (gfloat)31.0;
+            masks[3].mask     = 0x0;
+            masks[3].shiftin  = 0;
+            masks[3].max_value= (gfloat)0.0;
+            break;
+         default:
+            break;
+         }
     }
-  else if (Bitmap_File_Head.biSize <= 64) /* Probably OS/2 2.x */
+  else if (Bitmap_File_Head.biSize >= 56 && Bitmap_File_Head.biSize <= 64)
+    /* enhanced Windows format with bit masks */
     {
       if (!ReadOK (fd, buffer, Bitmap_File_Head.biSize - 4))
         {
@@ -271,8 +340,13 @@ ReadBMP (const gchar *name)
       Bitmap_Head.biYPels   =ToL (&buffer[0x18]);       /* 2A */
       Bitmap_Head.biClrUsed =ToL (&buffer[0x1C]);       /* 2E */
       Bitmap_Head.biClrImp  =ToL (&buffer[0x20]);       /* 32 */
-                                                        /* 36 */
-      Maps = 3;
+      Bitmap_Head.masks[0]  =ToL (&buffer[0x24]);       /* 36 */
+      Bitmap_Head.masks[1]  =ToL (&buffer[0x28]);       /* 3A */
+      Bitmap_Head.masks[2]  =ToL (&buffer[0x2C]);       /* 3E */
+      Bitmap_Head.masks[3]  =ToL (&buffer[0x30]);       /* 42 */
+
+      Maps = 4;
+      ReadChannelMasks (&Bitmap_Head.masks[0], masks, 4);
     }
   else
     {
@@ -286,7 +360,8 @@ ReadBMP (const gchar *name)
 
   /* There should be some colors used! */
 
-  ColormapSize = (Bitmap_File_Head.bfOffs - Bitmap_File_Head.biSize - 14) / Maps;
+  ColormapSize =
+    (Bitmap_File_Head.bfOffs - Bitmap_File_Head.biSize - 14) / Maps;
 
   if ((Bitmap_Head.biClrUsed == 0) && (Bitmap_Head.biBitCnt <= 8))
     ColormapSize = Bitmap_Head.biClrUsed = 1 << Bitmap_Head.biBitCnt;
@@ -302,6 +377,13 @@ ReadBMP (const gchar *name)
       return -1;
   }
 
+  if (Bitmap_Head.biWidth < 0)
+    {
+      g_message (_("'%s' is not a valid BMP file"),
+                  gimp_filename_to_utf8 (filename));
+      return -1;
+    }
+
   if (Bitmap_Head.biPlanes != 1) {
       g_message (_("Error reading BMP file header from '%s'"),
                   gimp_filename_to_utf8 (filename));
@@ -332,56 +414,32 @@ ReadBMP (const gchar *name)
           rowbytes);
 #endif
 
-  if (Bitmap_Head.biCompr == BI_BITFIELDS &&
-      Bitmap_Head.biBitCnt == 16)
-    {
-      masks = g_new (Bitmap_Channel, 3);
-      if (! ReadChannelMasks (fd, masks, 3))
-        {
-          g_message (_("'%s' is not a valid BMP file"),
-                      gimp_filename_to_utf8 (filename));
-          return -1;
-        }
-    }
-  else if (Bitmap_Head.biBitCnt == 16)
-    {
-      /* Default is 5.5.5 if no masks are given */
-      masks = g_new (Bitmap_Channel, 3);
-      masks[0].mask     = 0x7c00;
-      masks[0].shiftin  = 10;
-      masks[0].shiftout = 3;
-      masks[1].mask     = 0x03e0;
-      masks[1].shiftin  = 5;
-      masks[1].shiftout = 3;
-      masks[2].mask     = 0x001f;
-      masks[2].shiftin  = 0;
-      masks[2].shiftout = 3;
-    }
-  else
+  if (Bitmap_Head.biBitCnt <= 8)
     {
+#ifdef DEBUG
+      printf ("Colormap read\n");
+#endif
       /* Get the Colormap */
       if (!ReadColorMap (fd, ColorMap, ColormapSize, Maps, &Grey))
         return -1;
-      masks = NULL;
     }
 
-#ifdef DEBUG
-  printf ("Colormap read\n");
-#endif
-
   fseek (fd, Bitmap_File_Head.bfOffs, SEEK_SET);
 
   /* Get the Image and return the ID or -1 on error*/
   image_ID = ReadImage (fd,
-			Bitmap_Head.biWidth,
-			Bitmap_Head.biHeight,
-			ColorMap,
-			Bitmap_Head.biClrUsed,
-			Bitmap_Head.biBitCnt,
-			Bitmap_Head.biCompr,
-			rowbytes,
-			Grey,
-			masks);
+                        Bitmap_Head.biWidth,
+                        ABS (Bitmap_Head.biHeight),
+                        ColorMap,
+                        Bitmap_Head.biClrUsed,
+                        Bitmap_Head.biBitCnt,
+                        Bitmap_Head.biCompr,
+                        rowbytes,
+                        Grey,
+                        masks);
+
+  if (image_ID < 0)
+    return -1;
 
   if (Bitmap_Head.biXPels > 0 && Bitmap_Head.biYPels > 0)
     {
@@ -400,20 +458,23 @@ ReadBMP (const gchar *name)
       gimp_image_set_resolution (image_ID, xresolution, yresolution);
     }
 
+  if (Bitmap_Head.biHeight < 0)
+    gimp_image_flip (image_ID, GIMP_ORIENTATION_VERTICAL);
+
   return image_ID;
 }
 
 static gint32
 ReadImage (FILE     *fd,
-	   gint      width,
-	   gint      height,
-	   guchar    cmap[256][3],
-	   gint      ncols,
-	   gint      bpp,
-	   gint      compression,
-	   gint      rowbytes,
-	   gboolean  grey,
-	   const Bitmap_Channel *masks)
+           gint      width,
+           gint      height,
+           guchar    cmap[256][3],
+           gint      ncols,
+           gint      bpp,
+           gint      compression,
+           gint      rowbytes,
+           gboolean  grey,
+           const Bitmap_Channel *masks)
 {
   guchar             v, n;
   GimpPixelRgn       pixel_rgn;
@@ -426,9 +487,11 @@ ReadImage (FILE     *fd,
   guchar             gimp_cmap[768];
   gushort            rgb;
   glong              rowstride, channels;
-  gint               i, j, cur_progress, max_progress;
+  gint               i, i_max, j, cur_progress, max_progress;
+  gint               total_bytes_read;
   GimpImageBaseType  base_type;
   GimpImageType      image_type;
+  guint32            px32;
 
   if (! (compression == BI_RGB ||
       (bpp == 8 && compression == BI_RLE8) ||
@@ -440,7 +503,7 @@ ReadImage (FILE     *fd,
       return -1;
     }
 
-  /* Make a new image in the gimp */
+  /* Make a new image in GIMP */
 
   switch (bpp)
     {
@@ -448,9 +511,16 @@ ReadImage (FILE     *fd,
     case 24:
     case 16:
       base_type = GIMP_RGB;
-      image_type = GIMP_RGB_IMAGE;
-
-      channels = 3;
+      if (masks[3].mask != 0)
+      {
+         image_type = GIMP_RGBA_IMAGE;
+         channels = 4;
+      }
+      else
+      {
+         image_type = GIMP_RGB_IMAGE;
+         channels = 3;
+      }
       break;
 
     case 8:
@@ -471,10 +541,20 @@ ReadImage (FILE     *fd,
       break;
 
     default:
-      g_message (_("Unrecognized or invalid BMP compression format."));
+      g_message (_("Unsupported or invalid bitdepth."));
       return -1;
     }
 
+  if ((width < 0) || (width > GIMP_MAX_IMAGE_SIZE))
+    {
+      g_message (_("Unsupported or invalid image width: %d"), width);
+      return -1;
+    }
+  if ((height < 0) || (height > GIMP_MAX_IMAGE_SIZE))
+    {
+      g_message (_("Unsupported or invalid image height: %d"), height);
+      return -1;
+    }
   image = gimp_image_new (width, height, base_type);
   layer = gimp_layer_new (image, _("Background"),
                           width, height,
@@ -485,7 +565,10 @@ ReadImage (FILE     *fd,
   gimp_image_add_layer (image, layer, 0);
   drawable = gimp_drawable_get (layer);
 
-  dest      = g_malloc (drawable->width * drawable->height * channels);
+  /* use g_malloc0 to initialize the dest buffer so that unspecified
+     pixels in RLE bitmaps show up as the zeroth element in the palette.
+  */
+  dest      = g_malloc0 (drawable->width * drawable->height * channels);
   buffer    = g_malloc (rowbytes);
   rowstride = drawable->width * channels;
 
@@ -502,9 +585,12 @@ ReadImage (FILE     *fd,
             temp = dest + (ypos * rowstride);
             for (xpos= 0; xpos < width; ++xpos)
               {
-                *(temp++)= buffer[xpos * 4 + 2];
-                *(temp++)= buffer[xpos * 4 + 1];
-                *(temp++)= buffer[xpos * 4];
+                px32 = ToL(&buffer[xpos*4]);
+                *(temp++)= (guchar)((px32 & masks[0].mask) >> masks[0].shiftin);
+                *(temp++)= (guchar)((px32 & masks[1].mask) >> masks[1].shiftin);
+                *(temp++)= (guchar)((px32 & masks[2].mask) >> masks[2].shiftin);
+                if (channels > 3)
+                  *(temp++)= (guchar)((px32 & masks[3].mask) >> masks[3].shiftin);
               }
             if (ypos == 0)
               break;
@@ -514,6 +600,42 @@ ReadImage (FILE     *fd,
               gimp_progress_update ((gdouble) cur_progress /
                                     (gdouble) max_progress);
           }
+
+	if (channels == 4)
+	  {
+	    gboolean  has_alpha = FALSE;
+
+	    /* at least one pixel should have nonzero alpha */
+	    for (ypos = 0; ypos < height; ypos++)
+	      {
+		temp = dest + (ypos * rowstride);
+		for (xpos = 0; xpos < width; xpos++)
+		  {
+		    if (temp[3])
+		      {
+			has_alpha = TRUE;
+			break;
+		      }
+		    temp += 4;
+		  }
+		if (has_alpha)
+		  break;
+	      }
+
+	    /* workaround unwanted behaviour when all alpha pixels are zero */
+	    if (!has_alpha)
+	      {
+		for (ypos = 0; ypos < height; ypos++)
+		  {
+		    temp = dest + (ypos * rowstride);
+		    for (xpos = 0; xpos < width; xpos++)
+		      {
+			temp[3] = 255;
+			temp += 4;
+		      }
+		  }
+	      }
+	  }
       }
       break;
 
@@ -547,9 +669,11 @@ ReadImage (FILE     *fd,
             for (xpos= 0; xpos < width; ++xpos)
               {
                 rgb= ToS(&buffer[xpos * 2]);
-                *(temp++) = ((rgb & masks[0].mask) >> masks[0].shiftin) << masks[0].shiftout;
-                *(temp++) = ((rgb & masks[1].mask) >> masks[1].shiftin) << masks[1].shiftout;
-                *(temp++) = ((rgb & masks[2].mask) >> masks[2].shiftin) << masks[2].shiftout;
+                *(temp++) = (guchar)(((rgb & masks[0].mask) >> masks[0].shiftin) * 255.0 / masks[0].max_value + 0.5);
+                *(temp++) = (guchar)(((rgb & masks[1].mask) >> masks[1].shiftin) * 255.0 / masks[1].max_value + 0.5);
+                *(temp++) = (guchar)(((rgb & masks[2].mask) >> masks[2].shiftin) * 255.0 / masks[2].max_value + 0.5);
+                if (channels > 3)
+                  *(temp++) = (guchar)(((rgb & masks[3].mask) >> masks[3].shiftin) * 255.0 / masks[3].max_value + 0.5);
               }
             if (ypos == 0)
               break;
@@ -567,113 +691,142 @@ ReadImage (FILE     *fd,
     case 1:
       {
         if (compression == 0)
-	  {
-	    while (ReadOK (fd, &v, 1))
-	      {
-		for (i = 1; (i <= (8 / bpp)) && (xpos < width); i++, xpos++)
-		  {
-		    temp = dest + (ypos * rowstride) + (xpos * channels);
-		    *temp=( v & ( ((1<<bpp)-1) << (8-(i*bpp)) ) ) >> (8-(i*bpp));
-		    if (grey)
-		      *temp = cmap[*temp][0];
-		  }
-		if (xpos == width)
-		  {
-		    ReadOK (fd, buffer, rowbytes - 1 - (width * bpp - 1) / 8);
-            if (ypos == 0)
-              break;
-		    ypos--;
-		    xpos = 0;
+          /* no compression */
+          {
+            while (ReadOK (fd, &v, 1))
+              {
+                for (i = 1; (i <= (8 / bpp)) && (xpos < width); i++, xpos++)
+                  {
+                    temp = dest + (ypos * rowstride) + (xpos * channels);
+                    *temp=( v & ( ((1<<bpp)-1) << (8-(i*bpp)) ) ) >> (8-(i*bpp));
+                    if (grey)
+                      *temp = cmap[*temp][0];
+                  }
+                if (xpos == width)
+                  {
+                    fread(buffer, rowbytes - 1 - (width * bpp - 1) / 8, 1, fd);
+                    if (ypos == 0)
+                      break;
+                    ypos--;
+                    xpos = 0;
 
-		    cur_progress++;
-		    if ((cur_progress % 5) == 0)
-		      gimp_progress_update ((gdouble) cur_progress /
-					    (gdouble) max_progress);
-		  }
-		if (ypos < 0)
-		  break;
-	      }
-	    break;
-	  }
-	else
-	  {
-	    while (ypos >= 0 && xpos <= width)
-	      {
-		ReadOK (fd, buffer, 2);
-		if ((guchar) buffer[0] != 0)
-		  /* Count + Color - record */
-		  {
-		    for (j = 0; ((guchar) j < (guchar) buffer[0]) && (xpos < width);)
-		      {
+                    cur_progress++;
+                    if ((cur_progress % 5) == 0)
+                      gimp_progress_update ((gdouble) cur_progress /
+                                            (gdouble) max_progress);
+                  }
+                if (ypos < 0)
+                  break;
+              }
+            break;
+          }
+        else
+          {
+            /* compressed image (either RLE8 or RLE4) */
+            while (ypos >= 0 && xpos <= width)
+              {
+                if (!ReadOK (fd, buffer, 2))
+                  {
+                    g_message (_("The bitmap ends unexpectedly."));
+                    break;
+                  }
+
+                if ((guchar) buffer[0] != 0)
+                  /* Count + Color - record */
+                  {
+                    /* encoded mode run -
+                         buffer[0] == run_length
+                         buffer[1] == pixel data
+                    */
+                    for (j = 0;
+                         ((guchar) j < (guchar) buffer[0]) && (xpos < width);)
+                      {
 #ifdef DEBUG2
-			printf("%u %u | ",xpos,width);
+                        printf("%u %u | ",xpos,width);
 #endif
-			for (i = 1;
-			     ((i <= (8 / bpp)) &&
-			      (xpos < width) &&
-			      ((guchar) j < (unsigned char) buffer[0]));
-			     i++, xpos++, j++)
-			  {
-			    temp = dest + (ypos * rowstride) + (xpos * channels);
-			    *temp = (buffer[1] & (((1<<bpp)-1) << (8 - (i * bpp)))) >> (8 - (i * bpp));
-			    if (grey)
-			      *temp = cmap[*temp][0];
-			  }
-		      }
-		  }
-		if (((guchar) buffer[0] == 0) && ((guchar) buffer[1] > 2))
-		  /* uncompressed record */
-		  {
-		    n = buffer[1];
-		    for (j = 0; j < n; j += (8 / bpp))
-		      {
-			ReadOK (fd, &v, 1);
-			i = 1;
-			while ((i <= (8 / bpp)) && (xpos < width))
-			  {
-			    temp = dest + (ypos * rowstride) + (xpos * channels);
-			    *temp = (v & (((1<<bpp)-1) << (8-(i*bpp)))) >> (8-(i*bpp));
-			    if (grey)
-			      *temp = cmap[*temp][0];
-			    i++;
-			    xpos++;
-			  }
-		      }
+                        for (i = 1;
+                             ((i <= (8 / bpp)) &&
+                              (xpos < width) &&
+                              ((guchar) j < (unsigned char) buffer[0]));
+                             i++, xpos++, j++)
+                          {
+                            temp = dest + (ypos * rowstride) + (xpos * channels);
+                            *temp = (buffer[1] &
+                                     (((1<<bpp)-1) << (8 - (i * bpp)))) >> (8 - (i * bpp));
+                            if (grey)
+                              *temp = cmap[*temp][0];
+                          }
+                      }
+                  }
+                if (((guchar) buffer[0] == 0) && ((guchar) buffer[1] > 2))
+                  /* uncompressed record */
+                  {
+                    n = buffer[1];
+                    total_bytes_read = 0;
+                    for (j = 0; j < n; j += (8 / bpp))
+                      {
+                        /* read the next byte in the record */
+                        if (!ReadOK (fd, &v, 1))
+                          {
+                            g_message (_("The bitmap ends unexpectedly."));
+                            break;
+                          }
+                        total_bytes_read++;
 
-		    if ((n % 2) && (bpp==4))
-		      n++;
+                        /* read all pixels from that byte */
+                        i_max = 8 / bpp;
+                        if (n - j < i_max)
+                          {
+                            i_max = n - j;
+                          }
 
-		    if ((n / (8 / bpp)) % 2)
-		      ReadOK (fd, &v, 1);
+                        i = 1;
+                        while ((i <= i_max) && (xpos < width))
+                          {
+                            temp =
+                              dest + (ypos * rowstride) + (xpos * channels);
+                            *temp = (v >> (8-(i*bpp))) & ((1<<bpp)-1);
+                            if (grey)
+                              *temp = cmap[*temp][0];
+                            i++;
+                            xpos++;
+                          }
+                      }
 
-		    /*if odd(x div (8 div bpp )) then blockread(f,z^,1);*/
-		  }
-		if (((guchar) buffer[0] == 0) && ((guchar) buffer[1]==0))
-		  /* Line end */
-		  {
-		    ypos--;
-		    xpos = 0;
+                    /* absolute mode runs are padded to 16-bit alignment */
+                    if (total_bytes_read % 2)
+                      fread(&v, 1, 1, fd);
+                  }
+                if (((guchar) buffer[0] == 0) && ((guchar) buffer[1]==0))
+                  /* Line end */
+                  {
+                    ypos--;
+                    xpos = 0;
 
-		    cur_progress++;
-		    if ((cur_progress % 5) == 0)
-		      gimp_progress_update ((gdouble) cur_progress /
-					    (gdouble)  max_progress);
-		  }
-		if (((guchar) buffer[0]==0) && ((guchar) buffer[1]==1))
-		  /* Bitmap end */
-		  {
-		    break;
-		  }
-		if (((guchar) buffer[0]==0) && ((guchar) buffer[1]==2))
-		  /* Deltarecord */
-		  {
-		    ReadOK (fd, buffer, 2);
-		    xpos += (guchar) buffer[0];
-		    ypos -= (guchar) buffer[1];
-		  }
-	      }
-	    break;
-	  }
+                    cur_progress++;
+                    if ((cur_progress % 5) == 0)
+                      gimp_progress_update ((gdouble) cur_progress /
+                                            (gdouble)  max_progress);
+                  }
+                if (((guchar) buffer[0]==0) && ((guchar) buffer[1]==1))
+                  /* Bitmap end */
+                  {
+                    break;
+                  }
+                if (((guchar) buffer[0]==0) && ((guchar) buffer[1]==2))
+                  /* Deltarecord */
+                  {
+                    if (!ReadOK (fd, buffer, 2))
+                      {
+                        g_message (_("The bitmap ends unexpectedly."));
+                        break;
+                      }
+                    xpos += (guchar) buffer[0];
+                    ypos -= (guchar) buffer[1];
+                  }
+              }
+            break;
+          }
       }
       break;
 
@@ -686,17 +839,17 @@ ReadImage (FILE     *fd,
   if (bpp <= 8)
     for (i = 0, j = 0; i < ncols; i++)
       {
-	gimp_cmap[j++] = cmap[i][0];
-	gimp_cmap[j++] = cmap[i][1];
-	gimp_cmap[j++] = cmap[i][2];
+        gimp_cmap[j++] = cmap[i][0];
+        gimp_cmap[j++] = cmap[i][1];
+        gimp_cmap[j++] = cmap[i][2];
       }
 
   gimp_progress_update (1);
 
   gimp_pixel_rgn_init (&pixel_rgn, drawable,
-		       0, 0, drawable->width, drawable->height, TRUE, FALSE);
+                       0, 0, drawable->width, drawable->height, TRUE, FALSE);
   gimp_pixel_rgn_set_rect (&pixel_rgn, dest,
-			   0, 0, drawable->width, drawable->height);
+                           0, 0, drawable->width, drawable->height);
 
   if ((!grey) && (bpp<= 8))
     gimp_image_set_colormap (image, gimp_cmap, ncols);
