@@ -1,4 +1,4 @@
-/* The GIMP -- an image manipulation program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
  * Apply lens plug-in --- makes your selected part of the image look like it
@@ -39,14 +39,15 @@
 
 #include "config.h"
 
-#include <gtk/gtk.h>
-
 #include <libgimp/gimp.h>
 #include <libgimp/gimpui.h>
 
 #include "libgimp/stdplugins-intl.h"
 
-#define ENTRY_WIDTH 100
+
+#define PLUG_IN_PROC    "plug-in-applylens"
+#define PLUG_IN_BINARY  "apply_lens"
+
 
 /* Declare local functions.
  */
@@ -62,7 +63,7 @@ static void      drawlens    (GimpDrawable *drawable,
 static gboolean  lens_dialog (GimpDrawable *drawable);
 
 
-GimpPlugInInfo PLUG_IN_INFO =
+const GimpPlugInInfo PLUG_IN_INFO =
 {
   NULL,  /* init_proc  */
   NULL,  /* quit_proc  */
@@ -72,10 +73,10 @@ GimpPlugInInfo PLUG_IN_INFO =
 
 typedef struct
 {
-  gdouble refraction;
-  gint    keep_surr;
-  gint    use_bkgr;
-  gint    set_transparent;
+  gdouble  refraction;
+  gboolean keep_surr;
+  gboolean use_bkgr;
+  gboolean set_transparent;
 } LensValues;
 
 static LensValues lvals =
@@ -92,20 +93,21 @@ MAIN ()
 static void
 query (void)
 {
-  static GimpParamDef args[] =
+  static const GimpParamDef args[] =
   {
-    { GIMP_PDB_INT32, "run_mode", "Interactive, non-interactive" },
-    { GIMP_PDB_IMAGE, "image", "Input image (unused)" },
-    { GIMP_PDB_DRAWABLE, "drawable", "Input drawable" },
-    { GIMP_PDB_FLOAT, "refraction", "Lens refraction index" },
-    { GIMP_PDB_INT32, "keep_surroundings", "Keep lens surroundings" },
-    { GIMP_PDB_INT32, "set_background", "Set lens surroundings to bkgr value" },
-    { GIMP_PDB_INT32, "set_transparent", "Set lens surroundings transparent" }
+    { GIMP_PDB_INT32,    "run-mode",          "Interactive, non-interactive" },
+    { GIMP_PDB_IMAGE,    "image",             "Input image (unused)" },
+    { GIMP_PDB_DRAWABLE, "drawable",          "Input drawable" },
+    { GIMP_PDB_FLOAT,    "refraction",        "Lens refraction index" },
+    { GIMP_PDB_INT32,    "keep-surroundings", "Keep lens surroundings" },
+    { GIMP_PDB_INT32,    "set-background",    "Set lens surroundings to BG value" },
+    { GIMP_PDB_INT32,    "set-transparent",   "Set lens surroundings transparent" }
   };
 
-  gimp_install_procedure ("plug_in_applylens",
-                          "Apply a lens effect",
-                          "This plug-in uses Snell's law to draw an ellipsoid lens over the image",
+  gimp_install_procedure (PLUG_IN_PROC,
+                          N_("Simulate an elliptical lens over the image"),
+                          "This plug-in uses Snell's law to draw "
+                          "an ellipsoid lens over the image",
                           "Morten Eriksen",
                           "Morten Eriksen",
                           "1997",
@@ -115,8 +117,8 @@ query (void)
                           G_N_ELEMENTS (args), 0,
                           args, NULL);
 
-  gimp_plugin_menu_register ("plug_in_applylens",
-                             "<Image>/Filters/Glass Effects");
+  gimp_plugin_menu_register (PLUG_IN_PROC,
+                             "<Image>/Filters/Light and Shadow/Glass");
 }
 
 static void
@@ -135,18 +137,18 @@ run (const gchar      *name,
 
   run_mode = param[0].data.d_int32;
 
-  values[0].type = GIMP_PDB_STATUS;
+  values[0].type          = GIMP_PDB_STATUS;
   values[0].data.d_status = status;
 
   *nreturn_vals = 1;
-  *return_vals = values;
+  *return_vals  = values;
 
   drawable = gimp_drawable_get (param[2].data.d_drawable);
 
   switch(run_mode)
     {
     case GIMP_RUN_INTERACTIVE:
-      gimp_get_data ("plug_in_applylens", &lvals);
+      gimp_get_data (PLUG_IN_PROC, &lvals);
       if (!lens_dialog (drawable))
         return;
       break;
@@ -157,9 +159,9 @@ run (const gchar      *name,
 
       if (status == GIMP_PDB_SUCCESS)
         {
-          lvals.refraction = param[3].data.d_float;
-          lvals.keep_surr = param[4].data.d_int32;
-          lvals.use_bkgr = param[5].data.d_int32;
+          lvals.refraction      = param[3].data.d_float;
+          lvals.keep_surr       = param[4].data.d_int32;
+          lvals.use_bkgr        = param[5].data.d_int32;
           lvals.set_transparent = param[6].data.d_int32;
         }
 
@@ -168,7 +170,7 @@ run (const gchar      *name,
       break;
 
     case GIMP_RUN_WITH_LAST_VALS:
-      gimp_get_data ("plug_in_applylens", &lvals);
+      gimp_get_data (PLUG_IN_PROC, &lvals);
       break;
 
     default:
@@ -176,13 +178,13 @@ run (const gchar      *name,
     }
 
   gimp_tile_cache_ntiles (2 * (drawable->width / gimp_tile_width () + 1));
-  gimp_progress_init (_("Applying lens..."));
+  gimp_progress_init (_("Applying lens"));
   drawlens (drawable, NULL);
 
   if (run_mode != GIMP_RUN_NONINTERACTIVE)
     gimp_displays_flush ();
   if (run_mode == GIMP_RUN_INTERACTIVE)
-    gimp_set_data ("plug_in_applylens", &lvals, sizeof (LensValues));
+    gimp_set_data (PLUG_IN_PROC, &lvals, sizeof (LensValues));
 
   values[0].data.d_status = status;
 
@@ -386,17 +388,24 @@ lens_dialog (GimpDrawable *drawable)
   GtkObject *adj;
   gboolean   run;
 
-  gimp_ui_init ("apply_lens", FALSE);
+  gimp_ui_init (PLUG_IN_BINARY, FALSE);
 
-  dialog = gimp_dialog_new (_("Lens Effect"), "apply_lens",
+  dialog = gimp_dialog_new (_("Lens Effect"), PLUG_IN_BINARY,
                             NULL, 0,
-                            gimp_standard_help_func, "plug-in-applylens",
+                            gimp_standard_help_func, PLUG_IN_PROC,
 
                             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                             GTK_STOCK_OK,     GTK_RESPONSE_OK,
 
                             NULL);
 
+  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+                                           GTK_RESPONSE_OK,
+                                           GTK_RESPONSE_CANCEL,
+                                           -1);
+
+  gimp_window_set_transient (GTK_WINDOW (dialog));
+
   main_vbox = gtk_vbox_new (FALSE, 12);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), main_vbox);
@@ -473,10 +482,10 @@ lens_dialog (GimpDrawable *drawable)
 
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), spinbutton);
 
-  g_signal_connect (adj, "value_changed",
+  g_signal_connect (adj, "value-changed",
                     G_CALLBACK (gimp_double_adjustment_update),
                     &lvals.refraction);
-  g_signal_connect_swapped (adj, "value_changed",
+  g_signal_connect_swapped (adj, "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
 
