@@ -21,11 +21,9 @@
 
 #include "config.h"
 
-#include <stdlib.h>
 #include <string.h>
-#include <stdio.h>
 
-#include <gtk/gtk.h>
+#include <glib/gstdio.h>
 
 #include <libgimp/gimp.h>
 #include <libgimp/gimpui.h>
@@ -33,6 +31,10 @@
 #include "libgimp/stdplugins-intl.h"
 
 
+#define SAVE_PROC      "file-csource-save"
+#define PLUG_IN_BINARY "csource"
+
+
 typedef struct
 {
   gchar    *file_name;
@@ -62,7 +64,7 @@ static gboolean	run_save_dialog	(Config           *config);
 
 
 /* --- variables --- */
-GimpPlugInInfo PLUG_IN_INFO =
+const GimpPlugInInfo PLUG_IN_INFO =
 {
   NULL,  /* init_proc  */
   NULL,  /* quit_proc  */
@@ -90,16 +92,16 @@ MAIN ()
 static void
 query (void)
 {
-  static GimpParamDef save_args[] =
+  static const GimpParamDef save_args[] =
   {
-    { GIMP_PDB_INT32, "run_mode", "Interactive" },
-    { GIMP_PDB_IMAGE, "image", "Input image" },
-    { GIMP_PDB_DRAWABLE, "drawable", "Drawable to save" },
-    { GIMP_PDB_STRING, "filename", "The name of the file to save the image in" },
-    { GIMP_PDB_STRING, "raw_filename", "The name of the file to save the image in" }
+    { GIMP_PDB_INT32,    "run-mode",     "Interactive" },
+    { GIMP_PDB_IMAGE,    "image",        "Input image" },
+    { GIMP_PDB_DRAWABLE, "drawable",     "Drawable to save" },
+    { GIMP_PDB_STRING,   "filename",     "The name of the file to save the image in" },
+    { GIMP_PDB_STRING,   "raw-filename", "The name of the file to save the image in" }
   };
 
-  gimp_install_procedure ("file_csource_save",
+  gimp_install_procedure (SAVE_PROC,
                           "Dump image data in RGB(A) format for C source",
                           "CSource cannot be run non-interactively.",
                           "Tim Janik",
@@ -111,8 +113,8 @@ query (void)
                           G_N_ELEMENTS (save_args), 0,
                           save_args, NULL);
 
-  gimp_register_file_handler_mime ("file_csource_save", "text/x-csrc");
-  gimp_register_save_handler ("file_csource_save", "c", "");
+  gimp_register_file_handler_mime (SAVE_PROC, "text/x-csrc");
+  gimp_register_save_handler (SAVE_PROC, "c", "");
 }
 
 static void
@@ -138,7 +140,7 @@ run (const gchar      *name,
   values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
 
   if (run_mode == GIMP_RUN_INTERACTIVE &&
-      strcmp (name, "file_csource_save") == 0)
+      strcmp (name, SAVE_PROC) == 0)
     {
       gint32         image_ID    = param[1].data.d_int32;
       gint32         drawable_ID = param[2].data.d_int32;
@@ -146,7 +148,7 @@ run (const gchar      *name,
       gchar         *x;
       GimpImageType  drawable_type = gimp_drawable_type (drawable_ID);
 
-      gimp_get_data ("file_csource_save", &config);
+      gimp_get_data (SAVE_PROC, &config);
       config.prefixed_name = "gimp_image";
       config.comment       = NULL;
 
@@ -164,7 +166,7 @@ run (const gchar      *name,
 	}
       x = config.comment;
 
-      gimp_ui_init ("csource", FALSE);
+      gimp_ui_init (PLUG_IN_BINARY, FALSE);
       export = gimp_export_image (&image_ID, &drawable_ID, "C Source",
 				  (GIMP_EXPORT_CAN_HANDLE_RGB |
 				   GIMP_EXPORT_CAN_HANDLE_ALPHA ));
@@ -198,7 +200,7 @@ run (const gchar      *name,
 	    }
 	  else
 	    {
-	      gimp_set_data ("file_csource_save", &config, sizeof (config));
+	      gimp_set_data (SAVE_PROC, &config, sizeof (config));
 	    }
 	}
       else
@@ -382,7 +384,7 @@ save_image (Config *config,
   guint8 *img_buffer, *img_buffer_end;
   gchar *basename;
 
-  fp = fopen (config->file_name, "w");
+  fp = g_fopen (config->file_name, "w");
   if (!fp)
     return FALSE;
 
@@ -422,7 +424,7 @@ save_image (Config *config,
 		guint8 *d = data + x * drawable->bpp;
 		gdouble alpha = drawable_type == GIMP_RGBA_IMAGE ? d[3] : 0xff;
 
-		alpha *= config->opacity / 25600.0;
+		alpha *= config->opacity / 25500.0;
 		*(p++) = 0.5 + alpha * (gdouble) d[0];
 		*(p++) = 0.5 + alpha * (gdouble) d[1];
 		*(p++) = 0.5 + alpha * (gdouble) d[2];
@@ -632,15 +634,22 @@ run_save_dialog	(Config *config)
   GtkObject *adj;
   gboolean   run;
 
-  dialog = gimp_dialog_new (_("Save as C-Source"), "csource",
+  dialog = gimp_dialog_new (_("Save as C-Source"), PLUG_IN_BINARY,
                             NULL, 0,
-			    gimp_standard_help_func, "file-csource-save",
+			    gimp_standard_help_func, SAVE_PROC,
 
 			    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			    GTK_STOCK_OK,     GTK_RESPONSE_OK,
+			    GTK_STOCK_SAVE,   GTK_RESPONSE_OK,
 
 			    NULL);
 
+  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+                                           GTK_RESPONSE_OK,
+                                           GTK_RESPONSE_CANCEL,
+                                           -1);
+
+  gimp_window_set_transient (GTK_WINDOW (dialog));
+
   vbox = gtk_vbox_new (FALSE, 12);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), vbox);
@@ -742,7 +751,7 @@ run_save_dialog	(Config *config)
 			      config->opacity, 0, 100, 1, 10, 1,
 			      TRUE, 0, 0,
 			      NULL, NULL);
-  g_signal_connect (adj, "value_changed",
+  g_signal_connect (adj, "value-changed",
                     G_CALLBACK (gimp_double_adjustment_update),
                     &config->opacity);
 
