@@ -1,8 +1,8 @@
-/* LIC 0.14 -- image filter plug-in for The Gimp program
+/* LIC 0.14 -- image filter plug-in for GIMP
  * Copyright (C) 1996 Tom Bech
  *
  * E-mail: tomb@gimp.org
- * You can contact the original The Gimp authors at gimp@xcf.berkeley.edu
+ * You can contact the original GIMP authors at gimp@xcf.berkeley.edu
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,11 +36,6 @@
 
 #include "config.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-
-#include <gtk/gtk.h>
-
 #include <libgimp/gimp.h>
 #include <libgimp/gimpui.h>
 
@@ -54,7 +49,8 @@
 #define numx    40              /* Pseudo-random vector grid size */
 #define numy    40
 
-#define HELP_ID "plug-in-lic"
+#define PLUG_IN_PROC   "plug-in-lic"
+#define PLUG_IN_BINARY "lic"
 
 typedef enum
 {
@@ -446,8 +442,8 @@ rgb_to_hsl (GimpDrawable     *drawable,
 
   maxc = drawable->width * drawable->height;
 
-  gimp_pixel_rgn_init (&region, drawable, 0, 0,
-                       drawable->width, drawable->height, FALSE, FALSE);
+  gimp_pixel_rgn_init (&region, drawable, border_x1, border_y1,
+                       border_x2 - border_x1, border_y2 - border_y1, FALSE, FALSE);
 
   themap = g_new (guchar, maxc);
 
@@ -498,14 +494,14 @@ compute_lic (GimpDrawable *drawable,
   GimpPixelRgn src_rgn, dest_rgn;
 
   gimp_pixel_rgn_init (&src_rgn, drawable,
-                       0, 0,
-                       drawable->width,
-                       drawable->height, FALSE, FALSE);
+                       border_x1, border_y1,
+                       border_x2 - border_x1,
+                       border_y2 - border_y1, FALSE, FALSE);
 
   gimp_pixel_rgn_init (&dest_rgn, drawable,
-                       0, 0,
-                       drawable->width,
-                       drawable->height, TRUE, TRUE);
+                       border_x1, border_y1,
+                       border_x2 - border_x1,
+                       border_y2 - border_y1, TRUE, TRUE);
 
   for (ycount = 0; ycount < src_rgn.h; ycount++)
     {
@@ -564,11 +560,10 @@ compute_image (GimpDrawable *drawable)
 
   /* Get some useful info on the input drawable */
   /* ========================================== */
-
   gimp_drawable_mask_bounds (drawable->drawable_id,
                              &border_x1, &border_y1, &border_x2, &border_y2);
 
-  gimp_progress_init (_("Van Gogh (LIC)..."));
+  gimp_progress_init (_("Van Gogh (LIC)"));
 
   if (licvals.effect_convolve == 0)
     generatevectors ();
@@ -611,8 +606,8 @@ compute_image (GimpDrawable *drawable)
 
   gimp_drawable_flush (drawable);
   gimp_drawable_merge_shadow (drawable->drawable_id, TRUE);
-  gimp_drawable_update (drawable->drawable_id, 0, 0,
-                        drawable->width, drawable->height);
+  gimp_drawable_update (drawable->drawable_id, border_x1, border_y1,
+                        border_x2 - border_x1, border_y2 - border_y1);
 
   gimp_displays_flush ();
 }
@@ -641,17 +636,24 @@ create_main_dialog (void)
   gint       row;
   gboolean   run;
 
-  gimp_ui_init ("lic", TRUE);
+  gimp_ui_init (PLUG_IN_BINARY, TRUE);
 
-  dialog = gimp_dialog_new (_("Van Gogh (LIC)"), "lic",
+  dialog = gimp_dialog_new (_("Van Gogh (LIC)"), PLUG_IN_BINARY,
                             NULL, 0,
-                            gimp_standard_help_func, HELP_ID,
+                            gimp_standard_help_func, PLUG_IN_PROC,
 
                             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                             GTK_STOCK_OK,     GTK_RESPONSE_OK,
 
                             NULL);
 
+  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+                                           GTK_RESPONSE_OK,
+                                           GTK_RESPONSE_CANCEL,
+                                           -1);
+
+  gimp_window_set_transient (GTK_WINDOW (dialog));
+
   vbox = gtk_vbox_new (FALSE, 12);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), vbox);
@@ -711,7 +713,7 @@ create_main_dialog (void)
                               &licvals.effect_image_id);
 
   gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
-                             _("_Effect Image:"), 0.0, 0.5, combo, 2, TRUE);
+                             _("_Effect image:"), 0.0, 0.5, combo, 2, TRUE);
 
   table = gtk_table_new (5, 3, FALSE);
   gtk_table_set_col_spacings (GTK_TABLE (table), 6);
@@ -726,7 +728,7 @@ create_main_dialog (void)
                                      licvals.filtlen, 0.1, 64, 1.0, 8.0, 1,
                                      TRUE, 0, 0,
                                      NULL, NULL);
-  g_signal_connect (scale_data, "value_changed",
+  g_signal_connect (scale_data, "value-changed",
                     G_CALLBACK (gimp_double_adjustment_update),
                     &licvals.filtlen);
 
@@ -735,7 +737,7 @@ create_main_dialog (void)
                                      licvals.noisemag, 1, 5, 0.1, 1.0, 1,
                                      TRUE, 0, 0,
                                      NULL, NULL);
-  g_signal_connect (scale_data, "value_changed",
+  g_signal_connect (scale_data, "value-changed",
                     G_CALLBACK (gimp_double_adjustment_update),
                     &licvals.noisemag);
 
@@ -744,7 +746,7 @@ create_main_dialog (void)
                                      licvals.intsteps, 1, 40, 1.0, 5.0, 1,
                                      TRUE, 0, 0,
                                      NULL, NULL);
-  g_signal_connect (scale_data, "value_changed",
+  g_signal_connect (scale_data, "value-changed",
                     G_CALLBACK (gimp_double_adjustment_update),
                     &licvals.intsteps);
 
@@ -753,7 +755,7 @@ create_main_dialog (void)
                                      licvals.minv, -100, 0, 1, 10, 1,
                                      TRUE, 0, 0,
                                      NULL, NULL);
-  g_signal_connect (scale_data, "value_changed",
+  g_signal_connect (scale_data, "value-changed",
                     G_CALLBACK (gimp_double_adjustment_update),
                     &licvals.minv);
 
@@ -762,7 +764,7 @@ create_main_dialog (void)
                                      licvals.maxv, 0, 100, 1, 10, 1,
                                      TRUE, 0, 0,
                                      NULL, NULL);
-  g_signal_connect (scale_data, "value_changed",
+  g_signal_connect (scale_data, "value-changed",
                     G_CALLBACK (gimp_double_adjustment_update),
                     &licvals.maxv);
 
@@ -796,15 +798,15 @@ set_default_settings (void)
 static void
 query (void)
 {
-  static GimpParamDef args[] =
+  static const GimpParamDef args[] =
   {
-    { GIMP_PDB_INT32,    "run_mode", "Interactive"    },
+    { GIMP_PDB_INT32,    "run-mode", "Interactive"    },
     { GIMP_PDB_IMAGE,    "image",    "Input image"    },
     { GIMP_PDB_DRAWABLE, "drawable", "Input drawable" }
   };
 
-  gimp_install_procedure ("plug_in_lic",
-                          "Creates a Van Gogh effect (Line Integral Convolution)",
+  gimp_install_procedure (PLUG_IN_PROC,
+                          N_("Special effects that nobody understands"),
                           "No help yet",
                           "Tom Bech & Federico Mena Quintero",
                           "Tom Bech & Federico Mena Quintero",
@@ -815,7 +817,7 @@ query (void)
                           G_N_ELEMENTS (args), 0,
                           args, NULL);
 
-  gimp_plugin_menu_register ("plug_in_lic", "<Image>/Filters/Map");
+  gimp_plugin_menu_register (PLUG_IN_PROC, "<Image>/Filters/Artistic");
 }
 
 static void
@@ -848,7 +850,7 @@ run (const gchar      *name,
   /* Possibly retrieve data */
   /* ====================== */
 
-  gimp_get_data ("plug_in_lic", &licvals);
+  gimp_get_data (PLUG_IN_PROC, &licvals);
 
   /* Get the specified drawable */
   /* ========================== */
@@ -873,7 +875,7 @@ run (const gchar      *name,
                 if (create_main_dialog ())
                   compute_image (drawable);
 
-                gimp_set_data ("plug_in_lic", &licvals, sizeof (LicValues));
+                gimp_set_data (PLUG_IN_PROC, &licvals, sizeof (LicValues));
               break;
               case GIMP_RUN_WITH_LAST_VALS:
                 compute_image (drawable);
@@ -890,7 +892,7 @@ run (const gchar      *name,
   gimp_drawable_detach (drawable);
 }
 
-GimpPlugInInfo PLUG_IN_INFO =
+const GimpPlugInInfo PLUG_IN_INFO =
 {
   NULL,  /* init_proc  */
   NULL,  /* quit_proc  */
