@@ -1,7 +1,7 @@
-/* The GIMP -- an image manipulation program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * Polarize plug-in --- maps a rectangul to a circle or vice-versa
+ * Polarize plug-in --- maps a rectangle to a circle or vice-versa
  * Copyright (C) 1997 Daniel Dunbar
  * Email: ddunbar@diads.com
  * WWW:   http://millennium.diads.com/gimp/
@@ -57,30 +57,23 @@
 
 #include "config.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include <gtk/gtk.h>
-
 #include <libgimp/gimp.h>
 #include <libgimp/gimpui.h>
 
 #include "libgimp/stdplugins-intl.h"
 
+
 #define WITHIN(a, b, c) ((((a) <= (b)) && ((b) <= (c))) ? 1 : 0)
 
 
 /***** Magic numbers *****/
 
-#define PLUG_IN_NAME    "plug_in_polar_coords"
+#define PLUG_IN_PROC    "plug-in-polar-coords"
+#define PLUG_IN_BINARY  "polar"
 #define PLUG_IN_VERSION "July 1997, 0.5"
-#define HELP_ID         "plug-in-polar-coords"
 
-#define SCALE_WIDTH  200
-#define ENTRY_WIDTH   60
+#define SCALE_WIDTH     200
+#define ENTRY_WIDTH      60
 
 /***** Types *****/
 
@@ -91,7 +84,6 @@ typedef struct
   gboolean backwards;
   gboolean inverse;
   gboolean polrec;
-  gboolean preview;
 } polarize_vals_t;
 
 /***** Prototypes *****/
@@ -115,7 +107,7 @@ static void     dialog_update_preview   (GimpDrawable *drawable,
 
 /***** Variables *****/
 
-GimpPlugInInfo PLUG_IN_INFO =
+const GimpPlugInInfo PLUG_IN_INFO =
 {
   NULL,  /* init_proc  */
   NULL,  /* quit_proc  */
@@ -129,8 +121,7 @@ static polarize_vals_t pcvals =
   0.0,   /* angle */
   FALSE, /* backwards */
   TRUE,  /* inverse */
-  TRUE,  /* polar to rectangular? */
-  TRUE   /* preview */
+  TRUE   /* polar to rectangular? */
 };
 
 static gint img_width, img_height, img_has_alpha;
@@ -140,6 +131,8 @@ static gint sel_width, sel_height;
 static double cen_x, cen_y;
 static double scale_x, scale_y;
 
+static guchar back_color[4];
+
 /***** Functions *****/
 
 MAIN ()
@@ -147,9 +140,9 @@ MAIN ()
 static void
 query (void)
 {
-  static GimpParamDef args[] =
+  static const GimpParamDef args[] =
   {
-    { GIMP_PDB_INT32,    "run_mode",  "Interactive, non-interactive" },
+    { GIMP_PDB_INT32,    "run-mode",  "Interactive, non-interactive" },
     { GIMP_PDB_IMAGE,    "image",     "Input image"                  },
     { GIMP_PDB_DRAWABLE, "drawable",  "Input drawable"               },
     { GIMP_PDB_FLOAT,    "circle",    "Circle depth in %"            },
@@ -159,21 +152,21 @@ query (void)
     { GIMP_PDB_INT32,    "polrec",    "Polar to rectangular?"        }
   };
 
-  gimp_install_procedure (PLUG_IN_NAME,
-                          "Converts and image to and from polar coords",
+  gimp_install_procedure (PLUG_IN_PROC,
+                          N_("Convert image to or from polar coordinates"),
                           "Remaps and image from rectangular coordinates "
                           "to polar coordinates "
                           "or vice versa",
                           "Daniel Dunbar and Federico Mena Quintero",
                           "Daniel Dunbar and Federico Mena Quintero",
                           PLUG_IN_VERSION,
-                          N_("P_olar Coords..."),
+                          N_("P_olar Coordinates..."),
                           "RGB*, GRAY*",
                           GIMP_PLUGIN,
                           G_N_ELEMENTS (args), 0,
                           args, NULL);
 
-  gimp_plugin_menu_register (PLUG_IN_NAME, "<Image>/Filters/Distorts");
+  gimp_plugin_menu_register (PLUG_IN_PROC, "<Image>/Filters/Distorts");
 }
 
 static void
@@ -189,6 +182,7 @@ run (const gchar      *name,
   GimpPDBStatusType  status;
   GimpRunMode        run_mode;
   gdouble            xhsiz, yhsiz;
+  GimpRGB            background;
 
   status   = GIMP_PDB_SUCCESS;
   run_mode = param[0].data.d_int32;
@@ -209,7 +203,8 @@ run (const gchar      *name,
   img_height    = gimp_drawable_height (drawable->drawable_id);
   img_has_alpha = gimp_drawable_has_alpha (drawable->drawable_id);
 
-  gimp_drawable_mask_bounds (drawable->drawable_id, &sel_x1, &sel_y1, &sel_x2, &sel_y2);
+  gimp_drawable_mask_bounds (drawable->drawable_id,
+                             &sel_x1, &sel_y1, &sel_x2, &sel_y2);
 
   /* Calculate scaling parameters */
 
@@ -238,13 +233,19 @@ run (const gchar      *name,
       scale_y = 1.0;
     }
 
+  /* Get background color */
+  gimp_context_get_background (&background);
+  gimp_rgb_set_alpha (&background, 0.0);
+  gimp_drawable_get_color_uchar (drawable->drawable_id, &background,
+                                 back_color);
+
   /* See how we will run */
 
   switch (run_mode)
     {
     case GIMP_RUN_INTERACTIVE:
       /* Possibly retrieve data */
-      gimp_get_data (PLUG_IN_NAME, &pcvals);
+      gimp_get_data (PLUG_IN_PROC, &pcvals);
 
       /* Get information from the dialog */
       if (! polarize_dialog (drawable))
@@ -270,7 +271,7 @@ run (const gchar      *name,
 
     case GIMP_RUN_WITH_LAST_VALS:
       /* Possibly retrieve data */
-      gimp_get_data (PLUG_IN_NAME, &pcvals);
+      gimp_get_data (PLUG_IN_PROC, &pcvals);
       break;
 
     default:
@@ -295,7 +296,7 @@ run (const gchar      *name,
 
       /* Store data */
       if (run_mode == GIMP_RUN_INTERACTIVE)
-        gimp_set_data (PLUG_IN_NAME, &pcvals, sizeof (polarize_vals_t));
+        gimp_set_data (PLUG_IN_PROC, &pcvals, sizeof (polarize_vals_t));
     }
   else if (status == GIMP_PDB_SUCCESS)
     status = GIMP_PDB_EXECUTION_ERROR;
@@ -337,7 +338,7 @@ polarize_func (gint x,
       gint b;
       for (b = 0; b < bpp; b++)
         {
-          dest[b] = 255;
+          dest[b] = back_color[b];
         }
     }
 }
@@ -352,9 +353,11 @@ polarize (GimpDrawable *drawable)
   pft = gimp_pixel_fetcher_new (drawable, FALSE);
 
   gimp_context_get_background (&background);
+  gimp_rgb_set_alpha (&background, 0.0);
   gimp_pixel_fetcher_set_bg_color (pft, &background);
+  gimp_pixel_fetcher_set_edge_mode (pft, GIMP_PIXEL_FETCHER_EDGE_SMEAR);
 
-  gimp_progress_init (_("Polarizing..."));
+  gimp_progress_init (_("Polar coordinates"));
 
   iter = gimp_rgn_iterator_new (drawable, 0);
   gimp_rgn_iterator_dest (iter, polarize_func, pft);
@@ -584,24 +587,31 @@ polarize_dialog (GimpDrawable *drawable)
   GtkObject *adj;
   gboolean   run;
 
-  gimp_ui_init ("polar", TRUE);
+  gimp_ui_init (PLUG_IN_BINARY, TRUE);
 
-  dialog = gimp_dialog_new (_("Polarize"), "polar",
+  dialog = gimp_dialog_new (_("Polar Coordinates"), PLUG_IN_BINARY,
                             NULL, 0,
-                            gimp_standard_help_func, HELP_ID,
+                            gimp_standard_help_func, PLUG_IN_PROC,
 
                             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                             GTK_STOCK_OK,     GTK_RESPONSE_OK,
 
                             NULL);
 
+  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+                                           GTK_RESPONSE_OK,
+                                           GTK_RESPONSE_CANCEL,
+                                           -1);
+
+  gimp_window_set_transient (GTK_WINDOW (dialog));
+
   main_vbox = gtk_vbox_new (FALSE, 12);
   gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 12);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), main_vbox);
   gtk_widget_show (main_vbox);
 
   /* Preview */
-  preview = gimp_aspect_preview_new (drawable, &pcvals.preview);
+  preview = gimp_zoom_preview_new (drawable);
   gtk_box_pack_start_defaults (GTK_BOX (main_vbox), preview);
   gtk_widget_show (preview);
   g_signal_connect_swapped (preview, "invalidated",
@@ -621,10 +631,10 @@ polarize_dialog (GimpDrawable *drawable)
                               pcvals.circle, 0.0, 100.0, 1.0, 10.0, 2,
                               TRUE, 0, 0,
                               NULL, NULL);
-  g_signal_connect (adj, "value_changed",
+  g_signal_connect (adj, "value-changed",
                     G_CALLBACK (gimp_double_adjustment_update),
                     &pcvals.circle);
-  g_signal_connect_swapped (adj, "value_changed",
+  g_signal_connect_swapped (adj, "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
 
@@ -633,10 +643,10 @@ polarize_dialog (GimpDrawable *drawable)
                               pcvals.angle, 0.0, 359.0, 1.0, 15.0, 2,
                               TRUE, 0, 0,
                               NULL, NULL);
-  g_signal_connect (adj, "value_changed",
+  g_signal_connect (adj, "value-changed",
                     G_CALLBACK (gimp_double_adjustment_update),
                     &pcvals.angle);
-  g_signal_connect_swapped (adj, "value_changed",
+  g_signal_connect_swapped (adj, "value-changed",
                             G_CALLBACK (gimp_preview_invalidate),
                             preview);
 
@@ -713,79 +723,65 @@ static void
 dialog_update_preview (GimpDrawable *drawable,
                        GimpPreview  *preview)
 {
-  gdouble  left, right, bottom, top;
-  gdouble  dx, dy;
-  gdouble  px, py;
-  gdouble  cx = 0.0, cy = 0.0;
-  gint     ix, iy;
-  gint     x, y;
-  gint     width, height;
-  gint     bpp;
-  gdouble  scale_x, scale_y;
-  guchar  *p_ul, *i;
-  GimpRGB  background;
-  guchar   outside[4];
-  guchar  *buffer, *preview_cache;
-  guchar   k;
+  gdouble               cx, cy;
+  gint                  x, y;
+  gint                  sx, sy;
+  gint                  width, height;
+  guchar               *pixel;
+  guchar                outside[4];
+  GimpRGB               background;
+  guchar               *dest;
+  gint                  j;
+  gint                  bpp;
+  GimpPixelFetcher     *pft;
+  guchar                in_pixels[4][4];
+  guchar               *in_values[4];
+
+  for (j = 0; j < 4; j++)
+    in_values[j] = in_pixels[j];
+
+  pft = gimp_pixel_fetcher_new (drawable, FALSE);
 
   gimp_context_get_background (&background);
   gimp_rgb_set_alpha (&background, 0.0);
   gimp_drawable_get_color_uchar (drawable->drawable_id, &background, outside);
+  gimp_pixel_fetcher_set_bg_color (pft, &background);
+  gimp_pixel_fetcher_set_edge_mode (pft, GIMP_PIXEL_FETCHER_EDGE_SMEAR);
 
-  left   = sel_x1;
-  right  = sel_x2 - 1;
-  bottom = sel_y2 - 1;
-  top    = sel_y1;
-
-  gimp_preview_get_size (preview, &width, &height);
-  bpp = gimp_drawable_bpp (drawable->drawable_id);
-  preview_cache = gimp_drawable_get_thumbnail_data (drawable->drawable_id,
-                                                    &width, &height, &bpp);
-  dx = (right - left) / (width - 1);
-  dy = (bottom - top) / (height - 1);
-
-  scale_x = (double) width / (right - left + 1);
-  scale_y = (double) height / (bottom - top + 1);
-
-  py = top;
-
-  buffer = g_new (guchar, bpp * width * height);
-  p_ul = buffer;
+  dest = gimp_zoom_preview_get_source (GIMP_ZOOM_PREVIEW (preview),
+                                       &width, &height, &bpp);
+  pixel = dest;
 
   for (y = 0; y < height; y++)
     {
-      px = left;
-
       for (x = 0; x < width; x++)
         {
-          calc_undistorted_coords (px, py, &cx, &cy);
+          gimp_preview_untransform (preview, x, y, &sx, &sy);
+          if (calc_undistorted_coords ((gdouble)sx, (gdouble)sy,
+                                       &cx, &cy))
+            {
 
-          cx = (cx - left) * scale_x;
-          cy = (cy - top) * scale_y;
+              gimp_pixel_fetcher_get_pixel (pft, cx, cy, in_pixels[0]);
+              gimp_pixel_fetcher_get_pixel (pft, cx + 1, cy, in_pixels[1]);
+              gimp_pixel_fetcher_get_pixel (pft, cx, cy + 1, in_pixels[2]);
+              gimp_pixel_fetcher_get_pixel (pft, cx + 1, cy + 1, in_pixels[3]);
 
-          ix = (int) (cx + 0.5);
-          iy = (int) (cy + 0.5);
-
-          if ((ix >= 0) && (ix < width) &&
-              (iy >= 0) && (iy < height))
-            i = preview_cache + bpp * (width * iy + ix);
+              gimp_bilinear_pixels_8 (pixel, cx, cy, bpp,
+                                      img_has_alpha, in_values);
+            }
           else
-            i = outside;
+            {
+              for (j = 0; j < bpp; j++)
+                pixel[j] = outside[j];
+            }
 
-          for (k = 0; k < bpp; k ++)
-            p_ul[k] = i[k];
-
-          p_ul += bpp;
-
-          px += dx;
+          pixel += bpp;
         }
-
-      py += dy;
     }
 
-  gimp_preview_draw_buffer (preview, buffer, bpp * width);
+  gimp_pixel_fetcher_destroy (pft);
 
-  g_free (buffer);
-  g_free (preview_cache);
+  gimp_preview_draw_buffer (preview, dest, width * bpp);
+  g_free (dest);
 }
 
