@@ -1,4 +1,4 @@
-/* The GIMP -- an image manipulation program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  * SUN raster reading and writing code Copyright (C) 1996 Peter Kirchgessner
  * (email: peter@kirchgessner.net, WWW: http://www.kirchgessner.net)
@@ -34,19 +34,15 @@
  * V 1.94, PK, 12-Oct-97: No progress bars for non-interactive mode
  * V 1.95, nn, 20-Dec-97: Initialize some variable
  * V 1.96, PK, 21-Nov-99: Internationalization
+ * V 1.97, PK, 20-Dec-00: Recognize extensions .rs and .ras too
  */
-static char ident[] = "@(#) GIMP SunRaster file-plugin v1.96  21-Nov-99";
 
 #include "config.h"
 
-#include <stdio.h>
-#include <stdlib.h>
+#include <errno.h>
 #include <string.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
 
-#include <gtk/gtk.h>
+#include <glib/gstdio.h>
 
 #include <libgimp/gimp.h>
 #include <libgimp/gimpui.h>
@@ -54,6 +50,11 @@ static char ident[] = "@(#) GIMP SunRaster file-plugin v1.96  21-Nov-99";
 #include "libgimp/stdplugins-intl.h"
 
 
+#define LOAD_PROC      "file-sunras-load"
+#define SAVE_PROC      "file-sunras-save"
+#define PLUG_IN_BINARY "sunras"
+
+
 typedef int WRITE_FUN(void*,size_t,size_t,FILE*);
 
 typedef gulong  L_CARD32;
@@ -89,33 +90,40 @@ typedef struct
 /* Declare some local functions.
  */
 static void   query      (void);
-static void   run        (gchar   *name,
-                          gint     nparams,
-                          GimpParam  *param,
-                          gint    *nreturn_vals,
-                          GimpParam **return_vals);
+static void   run        (const gchar      *name,
+                          gint              nparams,
+                          const GimpParam  *param,
+                          gint             *nreturn_vals,
+                          GimpParam       **return_vals);
 
-static gint32 load_image (gchar   *filename);
-static gint   save_image (gchar   *filename,
-                          gint32   image_ID,
-                          gint32   drawable_ID);
+static gint32 load_image (const gchar      *filename);
+static gint   save_image (const gchar      *filename,
+                          gint32            image_ID,
+                          gint32            drawable_ID);
 
-static void set_color_table (gint32, L_SUNFILEHEADER *, unsigned char *);
+static void   set_color_table  (gint32, L_SUNFILEHEADER *, const guchar *);
+static gint32 create_new_image (const gchar   *filename,
+                                guint          width,
+                                guint          height,
+				GimpImageBaseType type,
+                                gint32        *layer_ID,
+                                GimpDrawable **drawable,
+				GimpPixelRgn  *pixel_rgn);
 
-static gint32 create_new_image (char *filename, guint width, guint height,
-				GimpImageBaseType type, gint32 *layer_ID, GimpDrawable **drawable,
-				GimpPixelRgn *pixel_rgn);
-
-static gint32 load_sun_d1   (char *, FILE *, L_SUNFILEHEADER *, unsigned char *);
-static gint32 load_sun_d8   (char *, FILE *, L_SUNFILEHEADER *, unsigned char *);
-static gint32 load_sun_d24  (char *, FILE *, L_SUNFILEHEADER *, unsigned char *);
-static gint32 load_sun_d32  (char *, FILE *, L_SUNFILEHEADER *, unsigned char *);
+static gint32 load_sun_d1   (const gchar *,
+                             FILE *, L_SUNFILEHEADER *, unsigned char *);
+static gint32 load_sun_d8   (const gchar *,
+                             FILE *, L_SUNFILEHEADER *, unsigned char *);
+static gint32 load_sun_d24  (const gchar *,
+                             FILE *, L_SUNFILEHEADER *, unsigned char *);
+static gint32 load_sun_d32  (const gchar *,
+                             FILE *, L_SUNFILEHEADER *, unsigned char *);
 
 static L_CARD32 read_card32 (FILE *, int *);
 
 static void write_card32    (FILE *, L_CARD32);
 
-static void byte2bit        (unsigned char *, int, unsigned char *, int);
+static void byte2bit        (unsigned char *, int, unsigned char *, gboolean);
 
 static void rle_startread   (FILE *);
 static int  rle_fread       (char *, int, int, FILE *);
@@ -141,14 +149,12 @@ static int read_msb_first = 1;
 static RLEBUF rlebuf;
 
 /* Dialog-handling */
-static gint   save_dialog              (void);
-static void   save_ok_callback         (GtkWidget *widget,
-                                        gpointer   data);
+static gboolean  save_dialog (void);
 
 /* Portability kludge */
 static int my_fwrite (void *ptr, int size, int nmemb, FILE *stream);
 
-GimpPlugInInfo PLUG_IN_INFO =
+const GimpPlugInInfo PLUG_IN_INFO =
 {
   NULL,  /* init_proc  */
   NULL,  /* quit_proc  */
@@ -160,121 +166,109 @@ GimpPlugInInfo PLUG_IN_INFO =
 /* Save info  */
 typedef struct
 {
-  gint  rle;  /*  rle or standard */
+  gboolean  rle;  /*  rle or standard */
 } SUNRASSaveVals;
 
-typedef struct
-{
-  gint  run;  /*  run  */
-} SUNRASSaveInterface;
-
 
 static SUNRASSaveVals psvals =
 {
   TRUE     /* rle */
 };
 
-static SUNRASSaveInterface psint =
-{
-  FALSE     /* run */
-};
-
 
 /* The run mode */
-static GimpRunModeType l_run_mode;
+static GimpRunMode l_run_mode;
 
 MAIN ()
 
 static void
 query (void)
 {
-  static GimpParamDef load_args[] =
+  static const GimpParamDef load_args[] =
   {
-    { GIMP_PDB_INT32,  "run_mode",      "Interactive, non-interactive" },
+    { GIMP_PDB_INT32,  "run-mode",      "Interactive, non-interactive" },
     { GIMP_PDB_STRING, "filename",      "The name of the file to load" },
-    { GIMP_PDB_STRING, "raw_filename",  "The name of the file to load" }
+    { GIMP_PDB_STRING, "raw-filename",  "The name of the file to load" }
   };
-  static gint nload_args = sizeof (load_args) / sizeof (load_args[0]);
 
-  static GimpParamDef load_return_vals[] =
+  static const GimpParamDef load_return_vals[] =
   {
     { GIMP_PDB_IMAGE,  "image",         "Output image" }
   };
-  static gint nload_return_vals = (sizeof (load_return_vals) /
-				   sizeof (load_return_vals[0]));
 
-  static GimpParamDef save_args[] =
+  static const GimpParamDef save_args[] =
   {
-    { GIMP_PDB_INT32,    "run_mode",     "Interactive, non-interactive" },
+    { GIMP_PDB_INT32,    "run-mode",     "Interactive, non-interactive" },
     { GIMP_PDB_IMAGE,    "image",        "Input image" },
     { GIMP_PDB_DRAWABLE, "drawable",     "Drawable to save" },
     { GIMP_PDB_STRING,   "filename",     "The name of the file to save the image in" },
-    { GIMP_PDB_STRING,   "raw_filename", "The name of the file to save the image in" },
+    { GIMP_PDB_STRING,   "raw-filename", "The name of the file to save the image in" },
     { GIMP_PDB_INT32,    "rle",          "Specify non-zero for rle output, zero for standard output" }
   };
-  static gint nsave_args = sizeof (save_args) / sizeof (save_args[0]);
 
-  gimp_install_procedure ("file_sunras_load",
+  gimp_install_procedure (LOAD_PROC,
                           "load file of the SunRaster file format",
                           "load file of the SunRaster file format",
                           "Peter Kirchgessner",
                           "Peter Kirchgessner",
                           "1996",
-                          "<Load>/SUNRAS",
+                          N_("SUN Rasterfile image"),
                           NULL,
                           GIMP_PLUGIN,
-                          nload_args, nload_return_vals,
+                          G_N_ELEMENTS (load_args),
+                          G_N_ELEMENTS (load_return_vals),
                           load_args, load_return_vals);
 
-  gimp_install_procedure ("file_sunras_save",
+  gimp_register_file_handler_mime (LOAD_PROC, "image/x-sun-raster");
+  gimp_register_magic_load_handler (LOAD_PROC,
+				    "im1,im8,im24,im32,rs,ras",
+				    "",
+				    "0,long,0x59a66a95");
+
+  gimp_install_procedure (SAVE_PROC,
                           "save file in the SunRaster file format",
                           "SUNRAS saving handles all image types except "
 			  "those with alpha channels.",
                           "Peter Kirchgessner",
                           "Peter Kirchgessner",
                           "1996",
-                          "<Save>/SUNRAS",
+                          N_("SUN Rasterfile image"),
                           "RGB, GRAY, INDEXED",
                           GIMP_PLUGIN,
-                          nsave_args, 0,
+                          G_N_ELEMENTS (save_args), 0,
                           save_args, NULL);
 
-  /* Magic information for sunras would be "0,long,0x59a66a95" */
-  gimp_register_magic_load_handler ("file_sunras_load", 
-				    "im1,im8,im24,im32", 
-				    "",
-				    "0,long,0x59a66a95");
-  gimp_register_save_handler       ("file_sunras_save", 
-				    "im1,im8,im24,im32", 
-				    "");
+  gimp_register_file_handler_mime (SAVE_PROC, "image/x-sun-raster");
+  gimp_register_save_handler (SAVE_PROC,
+                              "im1,im8,im24,im32,rs,ras", "");
 }
 
 
 static void
-run (gchar   *name,
-     gint     nparams,
-     GimpParam  *param,
-     gint    *nreturn_vals,
-     GimpParam **return_vals)
+run (const gchar      *name,
+     gint              nparams,
+     const GimpParam  *param,
+     gint             *nreturn_vals,
+     GimpParam       **return_vals)
 {
-  static GimpParam values[2];
-  GimpRunModeType  run_mode;
-  GimpPDBStatusType   status = GIMP_PDB_SUCCESS;
-  gint32        image_ID;
-  gint32        drawable_ID;
-  GimpExportReturnType export = GIMP_EXPORT_CANCEL;
+  static GimpParam  values[2];
+  GimpRunMode       run_mode;
+  GimpPDBStatusType status = GIMP_PDB_SUCCESS;
+  gint32            image_ID;
+  gint32            drawable_ID;
+  GimpExportReturn  export = GIMP_EXPORT_CANCEL;
 
   l_run_mode = run_mode = param[0].data.d_int32;
 
+  INIT_I18N ();
+
   *nreturn_vals = 1;
   *return_vals  = values;
   values[0].type          = GIMP_PDB_STATUS;
   values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
 
-  if (strcmp (name, "file_sunras_load") == 0)
+  if (strcmp (name, LOAD_PROC) == 0)
     {
-      INIT_I18N_UI();
-
       image_ID = load_image (param[1].data.d_string);
 
       if (image_ID != -1)
@@ -288,20 +282,18 @@ run (gchar   *name,
 	  status = GIMP_PDB_EXECUTION_ERROR;
 	}
     }
-  else if (strcmp (name, "file_sunras_save") == 0)
+  else if (strcmp (name, SAVE_PROC) == 0)
     {
-      INIT_I18N_UI();
-
       image_ID = param[1].data.d_int32;
       drawable_ID = param[2].data.d_int32;
 
-      /*  eventually export the image */ 
+      /*  eventually export the image */
       switch (run_mode)
 	{
 	case GIMP_RUN_INTERACTIVE:
 	case GIMP_RUN_WITH_LAST_VALS:
-	  gimp_ui_init ("sunras", FALSE);
-	  export = gimp_export_image (&image_ID, &drawable_ID, "SUNRAS", 
+	  gimp_ui_init (PLUG_IN_BINARY, FALSE);
+	  export = gimp_export_image (&image_ID, &drawable_ID, "SUNRAS",
 				      (GIMP_EXPORT_CAN_HANDLE_RGB |
 				       GIMP_EXPORT_CAN_HANDLE_GRAY |
 				       GIMP_EXPORT_CAN_HANDLE_INDEXED));
@@ -319,7 +311,7 @@ run (gchar   *name,
         {
         case GIMP_RUN_INTERACTIVE:
           /*  Possibly retrieve data  */
-          gimp_get_data ("file_sunras_save", &psvals);
+          gimp_get_data (SAVE_PROC, &psvals);
 
           /*  First acquire information with a dialog  */
           if (! save_dialog ())
@@ -340,7 +332,7 @@ run (gchar   *name,
 
         case GIMP_RUN_WITH_LAST_VALS:
           /*  Possibly retrieve data  */
-          gimp_get_data ("file_sunras_save", &psvals);
+          gimp_get_data (SAVE_PROC, &psvals);
           break;
 
         default:
@@ -352,8 +344,7 @@ run (gchar   *name,
 	  if (save_image (param[3].data.d_string, image_ID, drawable_ID))
 	    {
 	      /*  Store psvals data  */
-	      gimp_set_data ("file_sunras_save", &psvals,
-			     sizeof (SUNRASSaveVals));
+	      gimp_set_data (SAVE_PROC, &psvals, sizeof (SUNRASSaveVals));
 	    }
 	  else
 	    {
@@ -374,19 +365,19 @@ run (gchar   *name,
 
 
 static gint32
-load_image (gchar *filename)
+load_image (const gchar *filename)
 {
   gint32 image_ID;
   FILE *ifp;
-  char *temp = ident; /* Just to satisfy gcc/lint */
   L_SUNFILEHEADER sunhdr;
   guchar *suncolmap = NULL;
 
-  ifp = fopen (filename, "rb");
+  ifp = g_fopen (filename, "rb");
   if (!ifp)
     {
-      g_message (_("Can't open file for reading"));
-      return (-1);
+      g_message (_("Could not open '%s' for reading: %s"),
+                 gimp_filename_to_utf8 (filename), g_strerror (errno));
+      return -1;
     }
 
   read_msb_first = 1;   /* SUN raster is always most significant byte first */
@@ -394,7 +385,8 @@ load_image (gchar *filename)
   read_sun_header (ifp, &sunhdr);
   if (sunhdr.l_ras_magic != RAS_MAGIC)
     {
-      g_message (_("Can't open file as SUN-raster-file"));
+      g_message (_("Could not open '%s' as SUN-raster-file"),
+                 gimp_filename_to_utf8 (filename));
       fclose (ifp);
       return (-1);
     }
@@ -406,6 +398,12 @@ load_image (gchar *filename)
       return (-1);
     }
 
+  if ((sunhdr.l_ras_maplength < 0) || (sunhdr.l_ras_maplength > (256 * 3)))
+    {
+      g_message ("Map lengths greater than 256 entries are unsupported by GIMP.");
+      gimp_quit ();
+    }
+
   /* Is there a RGB colourmap ? */
   if ((sunhdr.l_ras_maptype == 1) && (sunhdr.l_ras_maplength > 0))
     {
@@ -424,7 +422,8 @@ load_image (gchar *filename)
 #endif
       if (sunhdr.l_ras_magic != RAS_MAGIC)
 	{
-	  g_message (_("Can't read color entries"));
+	  g_message (_("Could not read color entries from '%s'"),
+                     gimp_filename_to_utf8 (filename));
 	  fclose (ifp);
 	  return (-1);
 	}
@@ -435,14 +434,42 @@ load_image (gchar *filename)
       fseek (ifp, (sizeof (L_SUNFILEHEADER)/sizeof (L_CARD32))
 	     *4 + sunhdr.l_ras_maplength, SEEK_SET);
     }
-  
-  if (l_run_mode != GIMP_RUN_NONINTERACTIVE)
+
+  if (sunhdr.l_ras_width <= 0)
     {
-      temp = g_strdup_printf (_("Loading %s:"), filename);
-      gimp_progress_init (temp);
-      g_free (temp);
+      g_message (_("'%s':\nNo image width specified"),
+                 gimp_filename_to_utf8 (filename));
+      fclose (ifp);
+      return (-1);
     }
 
+  if (sunhdr.l_ras_width > GIMP_MAX_IMAGE_SIZE)
+    {
+      g_message (_("'%s':\nImage width is larger than GIMP can handle"),
+                 gimp_filename_to_utf8 (filename));
+      fclose (ifp);
+      return (-1);
+    }
+
+  if (sunhdr.l_ras_height <= 0)
+    {
+      g_message (_("'%s':\nNo image height specified"),
+                 gimp_filename_to_utf8 (filename));
+      fclose (ifp);
+      return (-1);
+    }
+
+  if (sunhdr.l_ras_height > GIMP_MAX_IMAGE_SIZE)
+    {
+      g_message (_("'%s':\nImage height is larger than GIMP can handle"),
+                 gimp_filename_to_utf8 (filename));
+      fclose (ifp);
+      return (-1);
+    }
+
+  gimp_progress_init_printf (_("Opening '%s'"),
+                             gimp_filename_to_utf8 (filename));
+
   switch (sunhdr.l_ras_depth)
     {
     case 1:    /* bitmap */
@@ -481,14 +508,13 @@ load_image (gchar *filename)
 
 
 static gint
-save_image (gchar  *filename,
-            gint32  image_ID,
-            gint32  drawable_ID)
+save_image (const gchar *filename,
+            gint32       image_ID,
+            gint32       drawable_ID)
 {
   FILE* ofp;
   GimpImageType drawable_type;
   gint retval;
-  char *temp;
 
   drawable_type = gimp_drawable_type (drawable_ID);
 
@@ -512,19 +538,16 @@ save_image (gchar  *filename,
     }
 
   /* Open the output file. */
-  ofp = fopen (filename, "wb");
+  ofp = g_fopen (filename, "wb");
   if (!ofp)
     {
-      g_message (_("Can't open file for writing"));
-      return (FALSE);
+      g_message (_("Could not open '%s' for writing: %s"),
+                 gimp_filename_to_utf8 (filename), g_strerror (errno));
+      return FALSE;
     }
 
-  if (l_run_mode != GIMP_RUN_NONINTERACTIVE)
-    {
-      temp = g_strdup_printf (_("Saving %s:"), filename);
-      gimp_progress_init (temp);
-      g_free (temp);
-    }
+  gimp_progress_init_printf (_("Saving '%s'"),
+                             gimp_filename_to_utf8 (filename));
 
   if (drawable_type == GIMP_INDEXED_IMAGE)
     retval = save_index (ofp,image_ID, drawable_ID, 0, (int)psvals.rle);
@@ -567,7 +590,7 @@ read_card32 (FILE *ifp,
 }
 
 
-static void 
+static void
 write_card32 (FILE     *ofp,
 	      L_CARD32  c)
 {
@@ -579,11 +602,11 @@ write_card32 (FILE     *ofp,
 
 
 /* Convert n bytes of 0/1 to a line of bits */
-static void 
-byte2bit (guchar *byteline,
-	  gint    width,
-	  guchar *bitline,
-	  gint    invert)
+static void
+byte2bit (guchar   *byteline,
+	  gint      width,
+	  guchar   *bitline,
+	  gboolean  invert)
 {
   register guchar bitval;
   guchar rest[8];
@@ -621,7 +644,7 @@ byte2bit (guchar *byteline,
 
 
 /* Start reading Runlength Encoded Data */
-static void 
+static void
 rle_startread (FILE *ifp)
 { /* Clear RLE-buffer */
   rlebuf.val = rlebuf.n = 0;
@@ -656,20 +679,20 @@ static gint
 rle_fgetc (FILE *ifp)
 {
   int flag, runcnt, runval;
-  
+
   if (rlebuf.n > 0)    /* Something in the buffer ? */
     {
       (rlebuf.n)--;
       return (rlebuf.val);
     }
-  
+
   /* Nothing in the buffer. We have to read something */
   if ((flag = getc (ifp)) < 0) return (-1);
   if (flag != 0x0080) return (flag);    /* Single byte run ? */
-  
+
   if ((runcnt = getc (ifp)) < 0) return (-1);
   if (runcnt == 0) return (0x0080);     /* Single 0x80 ? */
-  
+
   /* The run */
   if ((runval = getc (ifp)) < 0) return (-1);
   rlebuf.n = runcnt;
@@ -679,7 +702,7 @@ rle_fgetc (FILE *ifp)
 
 
 /* Start writing Runlength Encoded Data */
-static void 
+static void
 rle_startwrite (FILE *ofp)
 { /* Clear RLE-buffer */
   rlebuf.val = rlebuf.n = 0;
@@ -695,7 +718,7 @@ rle_fwrite (gchar *ptr,
 {
   int elem_write, cnt, val, err = 0;
   guchar *pixels = (unsigned char *)ptr;
-  
+
   for (elem_write = 0; elem_write < nelem; elem_write++)
     {
       for (cnt = 0; cnt < sz; cnt++)
@@ -715,15 +738,15 @@ rle_fputc (gint  val,
 	   FILE *ofp)
 {
   int retval;
-  
+
   if (rlebuf.n == 0)    /* Nothing in the buffer ? Save the value */
     {
       rlebuf.n = 1;
       return (rlebuf.val = val);
     }
-  
+
   /* Something in the buffer */
-  
+
   if (rlebuf.val == val)   /* Same value in the buffer ? */
     {
       (rlebuf.n)++;
@@ -735,12 +758,12 @@ rle_fputc (gint  val,
 	}
       return (val);
     }
-  
+
   /* Something different in the buffer ? Write out the run */
-  
+
   retval = rle_putrun (rlebuf.n, rlebuf.val, ofp);
   if (retval < 0) return (retval);
-  
+
   /* Save the new value */
   rlebuf.n = 1;
   return (rlebuf.val = val);
@@ -748,7 +771,7 @@ rle_fputc (gint  val,
 
 
 /* Write out a run with 0 < n < 257 */
-static gint 
+static gint
 rle_putrun (gint  n,
 	    gint  val,
 	    FILE *ofp)
@@ -774,14 +797,14 @@ rle_putrun (gint  n,
       else
 	retval = putc (val, ofp);
     }
-  
+
   return ((retval < 0) ? retval : val);
 }
 
 
 /* End writing Runlength Encoded Data */
-static void 
-rle_endwrite (FILE *ofp)    
+static void
+rle_endwrite (FILE *ofp)
 {
   if (rlebuf.n > 0)
     {
@@ -797,16 +820,16 @@ read_sun_header (FILE            *ifp,
 {
   int j, err;
   L_CARD32 *cp;
-  
+
   cp = (L_CARD32 *)sunhdr;
-  
+
   /* Read in all 32-bit values of the header and check for byte order */
   for (j = 0; j < sizeof (L_SUNFILEHEADER)/sizeof(sunhdr->l_ras_magic); j++)
     {
       *(cp++) = read_card32 (ifp, &err);
       if (err) break;
     }
-  
+
   if (err) sunhdr->l_ras_magic = 0;  /* Not a valid SUN-raster file */
 }
 
@@ -819,11 +842,11 @@ write_sun_header (FILE            *ofp,
 {
   int j, hdr_entries;
   L_CARD32 *cp;
-  
+
   hdr_entries = sizeof (L_SUNFILEHEADER)/sizeof(sunhdr->l_ras_magic);
-  
+
   cp = (L_CARD32 *)sunhdr;
-  
+
   /* Write out all 32-bit values of the header and check for byte order */
   for (j = 0; j < sizeof (L_SUNFILEHEADER)/sizeof(sunhdr->l_ras_magic); j++)
     {
@@ -840,14 +863,14 @@ read_sun_cols (FILE            *ifp,
                guchar          *colormap)
 {
   int ncols, err = 0;
-  
+
   /* Read in SUN-raster Colormap */
   ncols = sunhdr->l_ras_maplength / 3;
   if (ncols <= 0)
     err = 1;
   else
     err = (fread (colormap, 3, ncols, ifp) != ncols);
-  
+
   if (err) sunhdr->l_ras_magic = 0;  /* Not a valid SUN-raster file */
 }
 
@@ -860,7 +883,7 @@ write_sun_cols (FILE            *ofp,
                 guchar          *colormap)
 {
   int ncols;
-  
+
   ncols = sunhdr->l_ras_maplength / 3;
   fwrite (colormap, 3, ncols, ofp);
 }
@@ -871,19 +894,20 @@ write_sun_cols (FILE            *ofp,
 static void
 set_color_table (gint32           image_ID,
 		 L_SUNFILEHEADER *sunhdr,
-		 guchar          *suncolmap)
+		 const guchar    *suncolmap)
 {
-  int ncols, j;
-  guchar ColorMap[256*3];
+  guchar ColorMap[256 * 3];
+  gint   ncols, j;
 
   ncols = sunhdr->l_ras_maplength / 3;
-  if (ncols <= 0) return;
+  if (ncols <= 0)
+    return;
 
-  for (j = 0; j < ncols; j++)
+  for (j = 0; j < MIN (ncols, 256); j++)
     {
-      ColorMap[j*3] = suncolmap[j];
-      ColorMap[j*3+1] = suncolmap[j+ncols];
-      ColorMap[j*3+2] = suncolmap[j+2*ncols];
+      ColorMap[j * 3 + 0] = suncolmap[j];
+      ColorMap[j * 3 + 1] = suncolmap[j + ncols];
+      ColorMap[j * 3 + 2] = suncolmap[j + 2 * ncols];
     }
 
 #ifdef DEBUG
@@ -892,19 +916,20 @@ set_color_table (gint32           image_ID,
     printf ("%3d: 0x%02x 0x%02x 0x%02x\n", j,
 	    ColorMap[j*3], ColorMap[j*3+1], ColorMap[j*3+2]);
 #endif
-  gimp_image_set_cmap (image_ID, ColorMap, ncols);
+
+  gimp_image_set_colormap (image_ID, ColorMap, ncols);
 }
 
 
 /* Create an image. Sets layer_ID, drawable and rgn. Returns image_ID */
 static gint32
-create_new_image (gchar      *filename,
-                  guint       width,
-                  guint       height,
-                  GimpImageBaseType  type,
-                  gint32     *layer_ID,
-                  GimpDrawable **drawable,
-                  GimpPixelRgn  *pixel_rgn)
+create_new_image (const gchar        *filename,
+                  guint               width,
+                  guint               height,
+                  GimpImageBaseType   type,
+                  gint32             *layer_ID,
+                  GimpDrawable      **drawable,
+                  GimpPixelRgn       *pixel_rgn)
 {
   gint32 image_ID;
   GimpImageType gdtype;
@@ -914,7 +939,7 @@ create_new_image (gchar      *filename,
     case GIMP_RGB:
       gdtype = GIMP_RGB_IMAGE;
       break;
-    case GIMP_GRAY: 
+    case GIMP_GRAY:
       gdtype = GIMP_GRAY_IMAGE;
       break;
     case GIMP_INDEXED:
@@ -942,7 +967,7 @@ create_new_image (gchar      *filename,
 
 /* Load SUN-raster-file with depth 1 */
 static gint32
-load_sun_d1 (gchar           *filename,
+load_sun_d1 (const gchar     *filename,
              FILE            *ifp,
              L_SUNFILEHEADER *sunhdr,
              guchar          *suncolmap)
@@ -1020,7 +1045,7 @@ load_sun_d1 (gchar           *filename,
 
       scan_lines++;
 
-      if ((l_run_mode != GIMP_RUN_NONINTERACTIVE) && ((i % 20) == 0))
+      if ((i % 20) == 0)
 	gimp_progress_update ((double)(i+1) / (double)height);
 
       if ((scan_lines == tile_height) || ((i+1) == height))
@@ -1046,7 +1071,7 @@ load_sun_d1 (gchar           *filename,
 /* Load SUN-raster-file with depth 8 */
 
 static gint32
-load_sun_d8 (gchar           *filename,
+load_sun_d8 (const gchar     *filename,
              FILE            *ifp,
              L_SUNFILEHEADER *sunhdr,
              guchar          *suncolmap)
@@ -1111,7 +1136,7 @@ load_sun_d8 (gchar           *filename,
       dest += width;
       scan_lines++;
 
-      if ((l_run_mode != GIMP_RUN_NONINTERACTIVE) && ((i % 20) == 0))
+      if ((i % 20) == 0)
 	gimp_progress_update ((double)(i+1) / (double)height);
 
       if ((scan_lines == tile_height) || ((i+1) == height))
@@ -1136,7 +1161,7 @@ load_sun_d8 (gchar           *filename,
 
 /* Load SUN-raster-file with depth 24 */
 static gint32
-load_sun_d24 (gchar            *filename,
+load_sun_d24 (const gchar      *filename,
               FILE             *ifp,
               L_SUNFILEHEADER  *sunhdr,
               guchar           *suncolmap)
@@ -1192,7 +1217,7 @@ load_sun_d24 (gchar            *filename,
 
       scan_lines++;
 
-      if ((l_run_mode != GIMP_RUN_NONINTERACTIVE) && ((i % 20) == 0))
+      if ((i % 20) == 0)
 	gimp_progress_update ((double)(i+1) / (double)height);
 
       if ((scan_lines == tile_height) || ((i+1) == height))
@@ -1218,7 +1243,7 @@ load_sun_d24 (gchar            *filename,
 /* Load SUN-raster-file with depth 32 */
 
 static gint32
-load_sun_d32 (gchar           *filename,
+load_sun_d32 (const gchar     *filename,
               FILE            *ifp,
               L_SUNFILEHEADER *sunhdr,
               guchar          *suncolmap)
@@ -1287,7 +1312,7 @@ load_sun_d32 (gchar           *filename,
 
       scan_lines++;
 
-      if ((l_run_mode != GIMP_RUN_NONINTERACTIVE) && ((i % 20) == 0))
+      if ((i % 20) == 0)
 	gimp_progress_update ((double)(i+1) / (double)height);
 
       if ((scan_lines == tile_height) || ((i+1) == height))
@@ -1316,7 +1341,7 @@ save_index (FILE    *ofp,
             gint32  drawable_ID,
             gint    grey,
             gint    rle)
-{ 
+{
   int height, width, linepad, i, j;
   int ncols, bw, is_bw, is_wb, bpl;
   int tile_height;
@@ -1357,7 +1382,7 @@ save_index (FILE    *ofp,
     }
   else
     {
-      cmap = gimp_image_get_cmap (image_ID, &ncols);
+      cmap = gimp_image_get_colormap (image_ID, &ncols);
 
       for (j = 0; j < ncols; j++)
 	{
@@ -1427,7 +1452,7 @@ save_index (FILE    *ofp,
 	  if (linepad) (*write_fun) ((char *)&tmp, linepad, 1, ofp);
 	  src += width;
 
-	  if ((l_run_mode != GIMP_RUN_NONINTERACTIVE) && ((i % 20) == 0))
+	  if ((i % 20) == 0)
 	    gimp_progress_update ((double) i / (double) height);
 	}
     }
@@ -1440,24 +1465,24 @@ save_index (FILE    *ofp,
 	  if (linepad) (*write_fun) ((char *)&tmp, linepad, 1, ofp);
 	  src += width;
 
-	  if ((l_run_mode != GIMP_RUN_NONINTERACTIVE) && ((i % 20) == 0))
+	  if ((i % 20) == 0)
 	    gimp_progress_update ((double) i / (double) height);
 	}
     }
 
-  if (rle) 
+  if (rle)
     rle_endwrite (ofp);
 
   g_free (data);
 
-  if (bwline) 
+  if (bwline)
     g_free (bwline);
 
   gimp_drawable_detach (drawable);
 
   if (ferror (ofp))
     {
-      g_message (_("Write error occured"));
+      g_message (_("Write error occurred"));
       return (FALSE);
     }
   return (TRUE);
@@ -1530,8 +1555,8 @@ save_rgb (FILE   *ofp,
 	    }
 	  for (j = 0; j < linepad; j++)
 	    putc (0, ofp);
-	  
-	  if ((l_run_mode != GIMP_RUN_NONINTERACTIVE) && ((i % 20) == 0))
+
+	  if ((i % 20) == 0)
 	    gimp_progress_update ((double) i / (double) height);
 	}
     }
@@ -1553,7 +1578,7 @@ save_rgb (FILE   *ofp,
 	  for (j = 0; j < linepad; j++)
 	    rle_putc (0, ofp);
 
-	  if ((l_run_mode != GIMP_RUN_NONINTERACTIVE) && ((i % 20) == 0))
+	  if ((i % 20) == 0)
 	    gimp_progress_update ((double) i / (double) height);
 	}
 
@@ -1565,7 +1590,7 @@ save_rgb (FILE   *ofp,
 
   if (ferror (ofp))
     {
-      g_message (_("Write error occured"));
+      g_message (_("Write error occurred"));
       return (FALSE);
     }
   return (TRUE);
@@ -1575,62 +1600,57 @@ save_rgb (FILE   *ofp,
 
 /*  Save interface functions  */
 
-static gint
+static gboolean
 save_dialog (void)
 {
-  GtkWidget *dlg;
+  GtkWidget *dialog;
   GtkWidget *frame;
+  gboolean   run;
 
-  dlg = gimp_dialog_new (_("Save as SUNRAS"), "sunras",
-			 gimp_standard_help_func, "filters/sunras.html",
-			 GTK_WIN_POS_MOUSE,
-			 FALSE, TRUE, FALSE,
+  dialog = gimp_dialog_new (_("Save as SUNRAS"), PLUG_IN_BINARY,
+                            NULL, 0,
+                            gimp_standard_help_func, SAVE_PROC,
 
-			 _("OK"), save_ok_callback,
-			 NULL, NULL, NULL, TRUE, FALSE,
-			 _("Cancel"), gtk_widget_destroy,
-			 NULL, 1, NULL, FALSE, TRUE,
+                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                            GTK_STOCK_SAVE,   GTK_RESPONSE_OK,
 
-			 NULL);
+                            NULL);
 
-  gtk_signal_connect (GTK_OBJECT (dlg), "destroy",
-                      GTK_SIGNAL_FUNC (gtk_main_quit),
-                      NULL);
+  gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
+                                           GTK_RESPONSE_OK,
+                                           GTK_RESPONSE_CANCEL,
+                                           -1);
+
+  gimp_window_set_transient (GTK_WINDOW (dialog));
 
   /*  file save type  */
-  frame = gimp_radio_group_new2 (TRUE, _("Data Formatting"),
-				 gimp_radio_button_update,
-				 &psvals.rle, (gpointer) psvals.rle,
+  frame = gimp_int_radio_group_new (TRUE, _("Data Formatting"),
+                                    G_CALLBACK (gimp_radio_button_update),
+				    &psvals.rle, psvals.rle,
 
-				 _("RunLength Encoded"), (gpointer) TRUE, NULL,
-				 _("Standard"),          (gpointer) FALSE, NULL,
+				    _("RunLength Encoded"), TRUE,  NULL,
+				    _("Standard"),          FALSE, NULL,
 
-				 NULL);
-  gtk_container_set_border_width (GTK_CONTAINER (frame), 6);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), frame, TRUE, TRUE, 0);
+				    NULL);
+
+  gtk_container_set_border_width (GTK_CONTAINER (frame), 12);
+  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+                      frame, TRUE, TRUE, 0);
   gtk_widget_show (frame);
 
-  gtk_widget_show (dlg);
+  gtk_widget_show (dialog);
 
-  gtk_main ();
-  gdk_flush ();
+  run = (gimp_dialog_run (GIMP_DIALOG (dialog)) == GTK_RESPONSE_OK);
 
-  return psint.run;
+  gtk_widget_destroy (dialog);
+
+  return run;
 }
 
-static void
-save_ok_callback (GtkWidget *widget,
-                  gpointer   data)
-{
-  psint.run = TRUE;
-
-  gtk_widget_destroy (GTK_WIDGET (data));
-}
-
-static int 
-my_fwrite (void *ptr, 
-	   int   size, 
-	   int   nmemb, 
+static int
+my_fwrite (void *ptr,
+	   int   size,
+	   int   nmemb,
 	   FILE *stream)
 {
   return fwrite (ptr, size, nmemb, stream);
