@@ -1,5 +1,5 @@
 /*
- * This is a plug-in for the GIMP.
+ * This is a plug-in for GIMP.
  *
  * Generates clickable image maps.
  *
@@ -63,7 +63,7 @@ typedef struct {
    gboolean moved_first_time;
 } MoveCommand_t;
 
-Command_t* 
+Command_t*
 move_command_new(Preview_t *preview, Object_t *obj, gint x, gint y)
 {
    MoveCommand_t *command = g_new(MoveCommand_t, 1);
@@ -73,7 +73,7 @@ move_command_new(Preview_t *preview, Object_t *obj, gint x, gint y)
    command->obj = object_ref(obj);
    command->start_x = x;
    command->start_y = y;
-   object_get_dimensions(obj, &command->obj_x, &command->obj_y, 
+   object_get_dimensions(obj, &command->obj_x, &command->obj_y,
 			 &command->obj_width, &command->obj_height);
    command->obj_start_x = command->obj_x;
    command->obj_start_y = command->obj_y;
@@ -104,8 +104,8 @@ button_motion(GtkWidget *widget, GdkEventMotion *event, gpointer data)
    if (command->moved_first_time) {
       command->moved_first_time = FALSE;
       command->cursor = preview_set_cursor(command->preview, GDK_FLEUR);
-      gdk_gc_set_function(command->preferences->normal_gc, GDK_EQUIV);
-      gdk_gc_set_function(command->preferences->selected_gc, GDK_EQUIV);
+      gdk_gc_set_function(command->preferences->normal_gc, GDK_XOR);
+      gdk_gc_set_function(command->preferences->selected_gc, GDK_XOR);
       hide_url();
    }
 
@@ -149,7 +149,7 @@ button_release(GtkWidget *widget, GdkEventButton *event, gpointer data)
    command->obj_x -= command->obj_start_x;
    command->obj_y -= command->obj_start_y;
    if (command->obj_x || command->obj_y)
-      command_list_add(object_move_command_new(command->obj, command->obj_x, 
+      command_list_add(object_move_command_new(command->obj, command->obj_x,
 					       command->obj_y));
 
    /*   preview_thaw(); */
@@ -162,9 +162,9 @@ move_command_execute(Command_t *parent)
    GtkWidget *widget = command->preview->preview;
 
    /*   preview_freeze(); */
-   g_signal_connect(widget, "button_release_event", 
-                    G_CALLBACK (button_release), command);   
-   g_signal_connect(widget, "motion_notify_event", 
-                    G_CALLBACK (button_motion), command);   
+   g_signal_connect(widget, "button-release-event",
+                    G_CALLBACK (button_release), command);
+   g_signal_connect(widget, "motion-notify-event",
+                    G_CALLBACK (button_motion), command);
    return CMD_DESTRUCT;
 }
