@@ -1,4 +1,4 @@
-/* The GIMP -- an image manipulation program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,10 +19,13 @@
 #ifndef __SCRIPT_FU_SCRIPTS_H__
 #define __SCRIPT_FU_SCRIPTS_H__
 
-#include "script-fu-enums.h"
 
-void  script_fu_find_scripts  (void);
-LISP  script_fu_add_script    (LISP    a);
-void  script_fu_report_cc     (gchar  *command);
+void       script_fu_find_scripts  (const gchar *path);
+pointer    script_fu_add_script    (scheme *sc, pointer a);
+pointer    script_fu_add_menu      (scheme *sc, pointer a);
+void       script_fu_error_msg     (const gchar *command,
+                                    const gchar *msg);
+gchar   * script_fu_strescape      (const gchar *source);
+
 
 #endif /*  __SCRIPT_FU_SCRIPTS__  */
