@@ -1,4 +1,4 @@
-/* The GIMP -- an image manipulation program
+/* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
  *
  * GIMP Plug-in for Windows Icon files.
@@ -23,13 +23,9 @@
 #define __ICO_DIALOG_H__
 
 
-GtkWidget * ico_specs_dialog_new                 (gint       num_layers);
-void        ico_specs_dialog_add_icon            (GtkWidget *dialog,
-                                                  gint32     layer,
-                                                  gint       layer_num);
-void        ico_specs_dialog_update_icon_preview (GtkWidget *dialog,
-                                                  gint32     layer,
-                                                  gint       bpp);
-
+GtkWidget * ico_dialog_new                 (IcoSaveInfo *info);
+void        ico_dialog_add_icon            (GtkWidget *dialog,
+                                            gint32     layer,
+                                            gint       layer_num);
 
 #endif /* __ICO_DIALOG_H__ */
